@@ -1,9 +1,10 @@
 /*
- * Copyright 1999-2005 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -14,24 +15,28 @@
  * limitations under the License.
  */
 
-/* $Id$ */
+/* $Id: FOText.java 563951 2007-08-08 17:37:33Z vhennebert $ */
 
 package org.apache.fop.fo;
 
 // Java
+import java.awt.Color;
 import java.util.NoSuchElementException;
 
 // FOP
 import org.apache.fop.apps.FOPException;
-import org.apache.fop.datatypes.ColorType;
+import org.apache.fop.datatypes.Length;
 import org.apache.fop.fo.flow.Block;
-import org.apache.fop.fo.pagination.Root;
 import org.apache.fop.fo.properties.CommonFont;
 import org.apache.fop.fo.properties.CommonHyphenation;
 import org.apache.fop.fo.properties.CommonTextDecoration;
+import org.apache.fop.fo.properties.KeepProperty;
 import org.apache.fop.fo.properties.Property;
 import org.apache.fop.fo.properties.SpaceProperty;
 
+// SAX
+import org.xml.sax.Locator;
+
 /**
  * A text node (PCDATA) in the formatting object tree.
  *
@@ -39,9 +44,6 @@ import org.apache.fop.fo.properties.SpaceProperty;
  * onto references to the character data in this object
  * longer than the lifetime of the object itself, causing
  * excessive memory consumption and OOM errors.
- *
- * @author unascribed
- * @author <a href="mailto:mark-fop@inomial.com">Mark Lillywhite</a>
  */
 public class FOText extends FONode {
 
@@ -65,31 +67,29 @@ public class FOText extends FONode {
      * to be processed.
      *
      * This value is originally equal to ca.length, but becomes 
-     * decremented during between-word whitespace removal by the flow.Block class,  
-     * via the TextCharIterator.remove() method below.
+     * decremented during between-word whitespace removal by the 
+     * XMLWhiteSpaceHandler via the TextCharIterator.remove() 
+     * method below.
      */
     public int endIndex = 0;
 
-    // The value of properties relevant for character.
+    /** properties relevant for PCDATA */
+    /* TODO: these are basically always the same as the parent FObj or FObjMixed
+     *      so maybe those can be removed, and the accessors could  
+     *      dispatch the call to the parent?
+     */
     private CommonFont commonFont;
     private CommonHyphenation commonHyphenation;
-    private ColorType color;
+    private Color color;
+    private KeepProperty keepTogether;
     private Property letterSpacing;
     private SpaceProperty lineHeight;
+    private int whiteSpaceTreatment;
     private int whiteSpaceCollapse;
     private int textTransform;
     private Property wordSpacing;
     private int wrapOption;
-    // End of property values
-
-    /**
-     * Keeps track of the last FOText object created within the current
-     * block. This is used to create pointers between such objects.
-     * TODO: As soon as the control hierarchy is straightened out, this static
-     * variable needs to become an instance variable in some parent object,
-     * probably the page-sequence.
-     */
-    private static FOText lastFOTextProcessed = null;
+    private Length baselineShift;
 
     /**
      * Points to the previous FOText object created within the current
@@ -108,7 +108,7 @@ public class FOText extends FONode {
      * which FOText nodes are descendants of the same block.
      */
     private Block ancestorBlock = null;
-
+    
     /** Holds the text decoration values. May be null */
     private CommonTextDecoration textDecoration;
     
@@ -117,19 +117,47 @@ public class FOText extends FONode {
     private static final int IS_WORD_CHAR_MAYBE = 2;
 
     /**
-     *
-     * @param chars array of chars which contains the text in this object (may
-     * be a superset of the text in this object)
-     * @param start starting index into char[] for the text in this object
-     * @param end ending index into char[] for the text in this object
+     * Creates a now FO text node.
      * @param parent FONode that is the parent of this object
      */
-    public FOText(char[] chars, int start, int end, FONode parent) {
+    public FOText(FONode parent) {
         super(parent);
-        endIndex = end - start;
-        this.ca = new char[endIndex];
-        System.arraycopy(chars, start, ca, 0, endIndex);
-//      System.out.println("->" + new String(ca) + "<-");
+    }
+
+    /** @see org.apache.fop.fo.FONode */
+    protected void addCharacters(char[] data, int start, int end,
+            PropertyList list, Locator locator) throws FOPException {
+
+        int length = end - start;
+        int calength = 0;
+        char[] nca = null;
+        if (ca != null) {
+            calength = ca.length;
+            nca = new char[calength + length];
+            System.arraycopy(ca, 0, nca, 0, calength);
+        } else {
+            nca = new char[length];
+        }
+        System.arraycopy(data, start, nca, calength, length);
+        endIndex = nca.length;
+        this.ca = nca;
+     }
+
+    /**
+     * @see org.apache.fop.fo.FONode#clone(FONode, boolean)
+     */
+    public FONode clone(FONode parent, boolean removeChildren)
+            throws FOPException {
+        FOText ft = (FOText) super.clone(parent, removeChildren);
+        if (removeChildren) {
+            //not really removing, but just make sure the char array 
+            //pointed to is really a different one
+            if (ca != null) {
+                ft.ca = new char[ca.length];
+                System.arraycopy(ca, 0, ft.ca, 0, ca.length);
+            }
+        }
+        return ft;
     }
 
     /**
@@ -138,20 +166,23 @@ public class FOText extends FONode {
     public void bind(PropertyList pList) throws FOPException {
         commonFont = pList.getFontProps();
         commonHyphenation = pList.getHyphenationProps();
-        
-        color = pList.get(Constants.PR_COLOR).getColorType();
+        color = pList.get(Constants.PR_COLOR).getColor(getUserAgent());
+        keepTogether = pList.get(Constants.PR_KEEP_TOGETHER).getKeep();
         lineHeight = pList.get(Constants.PR_LINE_HEIGHT).getSpace();
         letterSpacing = pList.get(Constants.PR_LETTER_SPACING);
         whiteSpaceCollapse = pList.get(Constants.PR_WHITE_SPACE_COLLAPSE).getEnum();
+        whiteSpaceTreatment = pList.get(Constants.PR_WHITE_SPACE_TREATMENT).getEnum();
         textTransform = pList.get(Constants.PR_TEXT_TRANSFORM).getEnum();
         wordSpacing = pList.get(Constants.PR_WORD_SPACING);
         wrapOption = pList.get(Constants.PR_WRAP_OPTION).getEnum();
         textDecoration = pList.getTextDecorationProps();
+        baselineShift = pList.get(Constants.PR_BASELINE_SHIFT).getLength();
     }
 
-    protected void startOfNode() {
-        createBlockPointers();
+    /** @see org.apache.fop.fo.FONode#endOfNode() */
+    protected void endOfNode() throws FOPException {
         textTransform();
+        getFOEventHandler().characters(ca, startIndex, endIndex);
     }
 
     /**
@@ -189,38 +220,21 @@ public class FOText extends FONode {
     }
 
      /**
-     * This method is run as part of the Constructor, to create xref pointers to
-     * the previous FOText objects within the same Block
+     * This method is run as part of the ancestor Block's flushText(), to 
+     * create xref pointers to the previous FOText objects within the same Block
      */
-    private void createBlockPointers() {
-        // build pointers between the FOText objects withing the same Block
-        //
-        // find the ancestorBlock of the current node
-        FONode ancestorFONode = this;
-        while (this.ancestorBlock == null) {
-            ancestorFONode = ancestorFONode.parent;
-            Class myclass = ancestorFONode.getClass();
-            if (ancestorFONode instanceof org.apache.fop.fo.pagination.Title) {
-                return;
-            } else if (ancestorFONode instanceof Root) {
-                getLogger().warn("Unexpected: fo:text with no fo:block ancestor");
-                return;
-            } else if (ancestorFONode instanceof Block) {
-                this.ancestorBlock = (Block)ancestorFONode;
-            }
-        }
+    protected void createBlockPointers(Block ancestorBlock) {
+        this.ancestorBlock = ancestorBlock;
         // if the last FOText is a sibling, point to it, and have it point here
-        if (lastFOTextProcessed != null) {
-            if (lastFOTextProcessed.ancestorBlock == this.ancestorBlock) {
-                prevFOTextThisBlock = lastFOTextProcessed;
+        if (ancestorBlock.lastFOTextProcessed != null) {
+            if (ancestorBlock.lastFOTextProcessed.ancestorBlock 
+                    == this.ancestorBlock) {
+                prevFOTextThisBlock = ancestorBlock.lastFOTextProcessed;
                 prevFOTextThisBlock.nextFOTextThisBlock = this;
             } else {
                 prevFOTextThisBlock = null;
             }
         }
-        // save the current node in static field so the next guy knows where
-        // to look
-        lastFOTextProcessed = this;
         return;
     }
 
@@ -229,7 +243,8 @@ public class FOText extends FONode {
      * text-transform property.
      */
     private void textTransform() {
-        if (textTransform == Constants.EN_NONE) {
+        if (getFOEventHandler().inMarker() 
+                || textTransform == Constants.EN_NONE) {
             return;
         }
         for (int i = 0; i < endIndex; i++) {
@@ -302,8 +317,8 @@ public class FOText extends FONode {
         }
         // For now, we can't look at following FOText nodes
         if (offset > 0) {
-            return '\u0000';
-        }
+             return '\u0000';
+         }
         // Remaining case has the text in some previous FOText node
         boolean foundChar = false;
         char charToReturn = '\u0000';
@@ -374,10 +389,10 @@ public class FOText extends FONode {
                 */
                 return Character.toTitleCase(ca[i]);
             } else {
-                return Character.toLowerCase(ca[i]);
+                return ca[i];
             }
         default:
-            getLogger().warn("Invalid text-tranform value: " + textTransform);
+            log.warn("Invalid text-tranform value: " + textTransform);
             return ca[i];
         }
     }
@@ -480,7 +495,7 @@ public class FOText extends FONode {
         
         public boolean hasNext() {
            if (curIndex == 0) {
-//             System.out.println("->" + new String(ca) + "<-");
+//             log.debug("->" + new String(ca) + "<-");
           }
            return (curIndex < endIndex);
         }
@@ -499,16 +514,16 @@ public class FOText extends FONode {
             if (curIndex < endIndex && nextCharCalled < 2) {
                 startIndex++;
                 nextCharCalled = 0;
-//              System.out.println("removeA: " + new String(ca, startIndex, endIndex - startIndex));
+//              log.debug("removeA: " + new String(ca, startIndex, endIndex - startIndex));
             } else if (curIndex < endIndex) {
                 // copy from curIndex to end to curIndex-1
                 System.arraycopy(ca, curIndex, ca, curIndex - 1, 
                     endIndex - curIndex);
                 endIndex--;
                 curIndex--;
-//              System.out.println("removeB: " + new String(ca, startIndex, endIndex - startIndex));
+//              log.debug("removeB: " + new String(ca, startIndex, endIndex - startIndex));
             } else if (curIndex == endIndex) {
-//              System.out.println("removeC: " + new String(ca, startIndex, endIndex - startIndex));
+//              log.debug("removeC: " + new String(ca, startIndex, endIndex - startIndex));
                 endIndex--;
                 curIndex--;
             }
@@ -539,10 +554,17 @@ public class FOText extends FONode {
     /**
      * @return the "color" property.
      */
-    public ColorType getColor() {
+    public Color getColor() {
         return color;
     }
 
+    /** 
+     * @return the "keep-together" property.
+     */
+    public KeepProperty getKeepTogether() {
+        return keepTogether;
+    }
+
     /**
      * @return the "letter-spacing" property.
      */
@@ -557,6 +579,13 @@ public class FOText extends FONode {
         return lineHeight;
     }
     
+    /**
+     * @return the "white-space-treatment" property
+     */
+    public int getWhitespaceTreatment() {
+        return whiteSpaceTreatment;
+    }
+    
     /**
      * @return the "word-spacing" property.
      */
@@ -576,10 +605,34 @@ public class FOText extends FONode {
         return textDecoration; 
     }
 
+    /** @return the baseline-shift property */
+    public Length getBaseLineShift() {
+        return baselineShift;
+    }
+
+    /** @see java.lang.Object#toString() */
     public String toString() {
         StringBuffer sb = new StringBuffer(super.toString());
         sb.append(" (").append(ca).append(")");
         return sb.toString();
     }
-    
+ 
+    /** @see org.apache.fop.fo.FONode#getLocalName() */
+    public String getLocalName() {
+        return null;
+    }
+
+    /** @see org.apache.fop.fo.FONode#getNormalNamespacePrefix() */
+    public String getNormalNamespacePrefix() {
+        return null;
+    }
+
+    /** @see org.apache.fop.fo.FONode#gatherContextInfo() */
+    protected String gatherContextInfo() {
+        if (getLocator() != null) {
+            return super.gatherContextInfo();
+        } else {
+            return new String(ca).trim();
+        }
+    }    
 }
\ No newline at end of file
