@@ -1,9 +1,10 @@
 /*
- * Copyright 1999-2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -14,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id$ */
+/* $Id: CeilingFunction.java 554251 2007-07-07 20:13:41Z adelmelle $ */
  
 package org.apache.fop.fo.expr;
 
@@ -33,7 +34,7 @@ class CeilingFunction extends FunctionBase {
         if (dbl == null) {
             throw new PropertyException("Non number operand to ceiling function");
         }
-        return new NumberProperty(Math.ceil(dbl.doubleValue()));
+        return NumberProperty.getInstance(Math.ceil(dbl.doubleValue()));
     }
 
 }
