@@ -1,9 +1,10 @@
 /*
- * Copyright 1999-2004 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -14,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id$ */
+/* $Id: PDFShading.java 426576 2006-07-28 15:44:37Z jeremias $ */
  
 package org.apache.fop.pdf;
 
@@ -47,7 +48,7 @@ public class PDFShading extends PDFObject {
     /**
      * A ColorSpace representing the colorspace. "DeviceRGB" is an example.
      */
-    protected PDFColorSpace colorSpace = null;
+    protected PDFDeviceColorSpace colorSpace = null;
 
     /**
      * The background color. Since shading is opaque,
@@ -163,7 +164,7 @@ public class PDFShading extends PDFObject {
      * It's optional, the default is the identity matrix
      * @param theFunction The PDF Function that maps an (x,y) location to a color
      */
-    public PDFShading(int theShadingType, PDFColorSpace theColorSpace,
+    public PDFShading(int theShadingType, PDFDeviceColorSpace theColorSpace,
                       List theBackground, List theBBox,
                       boolean theAntiAlias, List theDomain,
                       List theMatrix, PDFFunction theFunction) {
@@ -201,7 +202,7 @@ public class PDFShading extends PDFObject {
      *                  and end colors past the start and end points
      * The default is [false, false]
      */
-    public PDFShading(int theShadingType, PDFColorSpace theColorSpace,
+    public PDFShading(int theShadingType, PDFDeviceColorSpace theColorSpace,
                       List theBackground, List theBBox,
                       boolean theAntiAlias, List theCoords,
                       List theDomain, PDFFunction theFunction,
@@ -241,7 +242,7 @@ public class PDFShading extends PDFObject {
      * @param theDecode List of Doubles see PDF 1.3 spec pages 303 to 312.
      * @param theFunction the PDFFunction
      */
-    public PDFShading(int theShadingType, PDFColorSpace theColorSpace,
+    public PDFShading(int theShadingType, PDFDeviceColorSpace theColorSpace,
                       List theBackground, List theBBox,
                       boolean theAntiAlias, int theBitsPerCoordinate,
                       int theBitsPerComponent, int theBitsPerFlag,
@@ -280,7 +281,7 @@ public class PDFShading extends PDFObject {
      * @param theVerticesPerRow number of vertices in each "row" of the lattice.
      * @param theFunction The PDFFunction that's mapped on to this shape
      */
-    public PDFShading(int theShadingType, PDFColorSpace theColorSpace,
+    public PDFShading(int theShadingType, PDFDeviceColorSpace theColorSpace,
                       List theBackground, List theBBox,
                       boolean theAntiAlias, int theBitsPerCoordinate,
                       int theBitsPerComponent, List theDecode,
@@ -341,7 +342,7 @@ public class PDFShading extends PDFObject {
             + "<< \n/ShadingType " + this.shadingType + " \n");
         if (this.colorSpace != null) {
             p.append("/ColorSpace /"
-                     + this.colorSpace.getColorSpacePDFString() + " \n");
+                     + this.colorSpace.getName() + " \n");
         }
 
         if (this.background != null) {
