@@ -1,9 +1,10 @@
 /*
- * Copyright 1999-2005 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *      http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -14,17 +15,18 @@
  * limitations under the License.
  */
 
-/* $Id$ */
+/* $Id: AbstractRenderer.java 542237 2007-05-28 14:31:24Z jeremias $ */
 
 package org.apache.fop.render;
 
 // Java
+import java.awt.Rectangle;
 import java.awt.geom.Rectangle2D;
 import java.io.IOException;
 import java.io.OutputStream;
-import java.util.Map;
 import java.util.List;
 import java.util.Iterator;
+import java.util.Set;
 
 // XML
 import org.w3c.dom.Document;
@@ -48,45 +50,41 @@ import org.apache.fop.area.RegionViewport;
 import org.apache.fop.area.RegionReference;
 import org.apache.fop.area.Trait;
 import org.apache.fop.area.OffDocumentItem;
+import org.apache.fop.area.inline.Character;
 import org.apache.fop.area.inline.Container;
 import org.apache.fop.area.inline.ForeignObject;
 import org.apache.fop.area.inline.Image;
 import org.apache.fop.area.inline.InlineArea;
+import org.apache.fop.area.inline.InlineBlockParent;
 import org.apache.fop.area.inline.InlineParent;
 import org.apache.fop.area.inline.Leader;
 import org.apache.fop.area.inline.Space;
 import org.apache.fop.area.inline.Viewport;
 import org.apache.fop.area.inline.TextArea;
-import org.apache.fop.area.inline.Character;
+import org.apache.fop.area.inline.WordArea;
+import org.apache.fop.area.inline.SpaceArea;
 import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.fo.Constants;
 import org.apache.fop.fonts.FontInfo;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
-// Avalon
-import org.apache.avalon.framework.configuration.Configurable;
-import org.apache.avalon.framework.configuration.Configuration;
-import org.apache.avalon.framework.configuration.ConfigurationException;
-
 /**
  * Abstract base class for all renderers. The Abstract renderer does all the
  * top level processing of the area tree and adds some abstract methods to
  * handle viewports. This keeps track of the current block and inline position.
  */
 public abstract class AbstractRenderer 
-         implements Renderer, Configurable, Constants {
+         implements Renderer, Constants {
 
+    /** logging instance */
+    protected static Log log = LogFactory.getLog("org.apache.fop.render");
+    
     /**
      * user agent
      */
-    protected FOUserAgent userAgent;
+    protected FOUserAgent userAgent = null;
 
-    /**
-     * logging instance
-     */
-    protected static Log logger = LogFactory.getLog("org.apache.fop.render");
-    
     /**
      * block progression position
      */
@@ -109,37 +107,35 @@ public abstract class AbstractRenderer
      */
     protected int containingIPPosition = 0;
 
+    /** the currently active PageViewport */
+    protected PageViewport currentPageViewport;
+    
+    private Set warnedXMLHandlers;
+    
     /**
-     * @see org.apache.avalon.framework.configuration.Configurable#configure(Configuration)
-     */
-    public void configure(Configuration conf) throws ConfigurationException {
-    }
-
-    /**
-     * Returns the Commons-Logging instance for this class
-     * @return  The Commons-Logging instance
-     */
-    protected Log getLogger() {
-        return logger;
-    }
-
-    /**
-     *  @see org.apache.fop.render.Renderer
+     *  @see org.apache.fop.render.Renderer#setupFontInfo(FontInfo)
      */
     public abstract void setupFontInfo(FontInfo fontInfo);
 
     /**
-     *  @see org.apache.fop.render.Renderer
+     *  @see org.apache.fop.render.Renderer#setUserAgent(FOUserAgent)
      */
     public void setUserAgent(FOUserAgent agent) {
         userAgent = agent;
     }
 
-    /** @see org.apache.fop.render.Renderer */
+    /**
+     *  @see org.apache.fop.render.Renderer#getUserAgent()
+     */
+    public FOUserAgent getUserAgent() {
+        return userAgent;
+    }
+
+    /** @see org.apache.fop.render.Renderer#startRenderer(OutputStream) */
     public void startRenderer(OutputStream outputStream)
         throws IOException { }
 
-    /** @see org.apache.fop.render.Renderer */
+    /** @see org.apache.fop.render.Renderer#stopRenderer() */
     public void stopRenderer()
         throws IOException { }
 
@@ -156,10 +152,25 @@ public abstract class AbstractRenderer
     }
 
     /**
-     * @see     org.apache.fop.render.Renderer
+     * @see     org.apache.fop.render.Renderer#processOffDocumentItem(OffDocumentItem)
      */
-    public void processOffDocumentItem(OffDocumentItem oDI) { }
+    public void processOffDocumentItem(OffDocumentItem odi) { }
 
+    /** @see org.apache.fop.render.Renderer#getGraphics2DAdapter() */
+    public Graphics2DAdapter getGraphics2DAdapter() {
+        return null;
+    }
+    
+    /** @see org.apache.fop.render.Renderer#getImageAdapter() */
+    public ImageAdapter getImageAdapter() {
+        return null;
+    }
+    
+    /** @return the current PageViewport or null, if none is active */
+    protected PageViewport getCurrentPageViewport() {
+        return this.currentPageViewport;
+    }
+    
     /**
      * Prepare a page for rendering. This is called if the renderer supports
      * out of order rendering. The renderer should prepare the page so that a
@@ -167,7 +178,7 @@ public abstract class AbstractRenderer
      * page should not be rendered. The page will be rendered at a later time
      * by the call to render page.
      *
-     * @see org.apache.fop.render.Renderer
+     * @see org.apache.fop.render.Renderer#preparePage(PageViewport)
      */
     public void preparePage(PageViewport page) { }
 
@@ -190,10 +201,10 @@ public abstract class AbstractRenderer
         StringBuffer sb = new StringBuffer();
         for (int count = 0; count < children.size(); count++) {
             InlineArea inline = (InlineArea) children.get(count);
-            if (inline instanceof Character) {
-                sb.append(((Character) inline).getChar());
-            } else if (inline instanceof TextArea) {
-                sb.append(((TextArea) inline).getTextArea());
+            //if (inline instanceof Character) {
+            //    sb.append(((Character) inline).getChar());
+            /*} else*/ if (inline instanceof TextArea) {
+                sb.append(((TextArea) inline).getText());
             } else if (inline instanceof InlineParent) {
                 sb.append(convertToString(
                         ((InlineParent) inline).getChildAreas()));
@@ -204,19 +215,24 @@ public abstract class AbstractRenderer
         return sb.toString();
     }
 
-    /** @see org.apache.fop.render.Renderer */
+    /** @see org.apache.fop.render.Renderer#startPageSequence(LineArea) */
     public void startPageSequence(LineArea seqTitle) {
         //do nothing
     }
 
     // normally this would be overriden to create a page in the
     // output
-    /** @see org.apache.fop.render.Renderer */
+    /** @see org.apache.fop.render.Renderer#renderPage(PageViewport) */
     public void renderPage(PageViewport page)
         throws IOException, FOPException {
 
-        Page p = page.getPage();
-        renderPageAreas(p);
+        this.currentPageViewport = page;
+        try {
+            Page p = page.getPage();
+            renderPageAreas(p);
+        } finally {
+            this.currentPageViewport = null;
+        }
     }
 
     /**
@@ -259,27 +275,35 @@ public abstract class AbstractRenderer
             currentBPPosition = 0;
             currentIPPosition = 0;
 
-            RegionReference region = port.getRegion();
+            RegionReference regionReference = port.getRegionReference();
             handleRegionTraits(port);
 
             //  shouldn't the viewport have the CTM
-            startVParea(region.getCTM());
+            startVParea(regionReference.getCTM(), port.isClip() ? view : null);
             // do after starting viewport area
-            if (region.getRegionClass() == FO_REGION_BODY) {
-                renderBodyRegion((BodyRegion) region);
+            if (regionReference.getRegionClass() == FO_REGION_BODY) {
+                renderBodyRegion((BodyRegion) regionReference);
             } else {
-                renderRegion(region);
+                renderRegion(regionReference);
             }
             endVParea();
         }
     }
 
     /**
-     * (todo) Description of the Method
+     * Establishes a new viewport area.
      *
-     * @param ctm  The coordinate transformation matrix to use
+     * @param ctm the coordinate transformation matrix to use
+     * @param clippingRect the clipping rectangle if the viewport should be clipping, 
+     *                     null if no clipping is performed.
      */
-    protected void startVParea(CTM ctm) { }
+    protected abstract void startVParea(CTM ctm, Rectangle2D clippingRect);
+
+    /**
+     * Signals exit from a viewport area. Subclasses can restore transformation matrices
+     * valid before the viewport area was started.
+     */
+    protected abstract void endVParea();
 
     /**
      * Handle the traits for a region
@@ -291,11 +315,6 @@ public abstract class AbstractRenderer
         // draw border and background
     }
 
-    /**
-     * (todo) Description of the Method
-     */
-    protected void endVParea() { }
-
     /**
      * Renders a region reference area.
      *
@@ -348,6 +367,7 @@ public abstract class AbstractRenderer
      * @param footnote  The footnote
      */
     protected void renderFootnote(Footnote footnote) {
+        currentBPPosition += footnote.getTop();
         List blocks = footnote.getChildAreas();
         if (blocks != null) {
             Block sep = footnote.getSeparator();
@@ -374,20 +394,25 @@ public abstract class AbstractRenderer
 
         Span span = null;
         List spans = mr.getSpans();
+        int saveBPPos = currentBPPosition;
+        int saveSpanBPPos = saveBPPos;
         for (int count = 0; count < spans.size(); count++) {
             span = (Span) spans.get(count);
-            int offset = (mr.getWidth()
-                    - (span.getColumnCount() - 1) * mr.getColumnGap())
-                    / span.getColumnCount() + mr.getColumnGap();
             for (int c = 0; c < span.getColumnCount(); c++) {
                 NormalFlow flow = (NormalFlow) span.getNormalFlow(c);
 
-                renderFlow(flow);
-                currentIPPosition += offset;
+                if (flow != null) {
+                    currentBPPosition = saveSpanBPPos;
+                    renderFlow(flow);
+                    currentIPPosition += flow.getIPD();
+                    currentIPPosition += mr.getColumnGap();
+                }
             }
             currentIPPosition = saveIPPos;
-            currentBPPosition += span.getHeight();
+            currentBPPosition = saveSpanBPPos + span.getHeight();
+            saveSpanBPPos = currentBPPosition;
         }
+        currentBPPosition = saveBPPos;
     }
 
     /**
@@ -428,11 +453,16 @@ public abstract class AbstractRenderer
             int saveIP = currentIPPosition;
             int saveBP = currentBPPosition;
 
+            Rectangle2D clippingRect = null;
+            if (bv.getClip()) {
+                clippingRect = new Rectangle(saveIP, saveBP, bv.getIPD(), bv.getBPD());
+            }
+            
             CTM ctm = bv.getCTM();
             currentIPPosition = 0;
             currentBPPosition = 0;
 
-            startVParea(ctm);
+            startVParea(ctm, clippingRect);
             handleBlockTraits(bv);
             renderBlocks(bv, children);
             endVParea();
@@ -500,6 +530,7 @@ public abstract class AbstractRenderer
                         + parent.getStartIndent() 
                         + line.getStartIndent();
                 renderLineArea(line);
+                //InlineArea child = (InlineArea) line.getInlineAreas().get(0);
                 currentBPPosition += line.getAllocBPD();
             }
             currentIPPosition = saveIP;
@@ -527,8 +558,9 @@ public abstract class AbstractRenderer
             int saveBP = currentBPPosition;
 
             if (block.getPositioning() == Block.ABSOLUTE) {
-                currentIPPosition = containingIPPosition + block.getXOffset();
-                currentBPPosition = containingBPPosition + block.getYOffset();
+                currentIPPosition += block.getXOffset();
+                currentBPPosition += block.getYOffset();
+                currentBPPosition += block.getSpaceBefore();
 
                 handleBlockTraits(block);
 
@@ -542,6 +574,7 @@ public abstract class AbstractRenderer
                 // relative blocks are offset
                 currentIPPosition += block.getXOffset();
                 currentBPPosition += block.getYOffset();
+                currentBPPosition += block.getSpaceBefore();
 
                 handleBlockTraits(block);
 
@@ -556,10 +589,6 @@ public abstract class AbstractRenderer
         }
     }
 
-    protected void renderTextDecoration(InlineArea area) {
-        //getLogger().debug("renderTextDecoration for " + area + " -> " + area.getTrait(Trait.UNDERLINE));
-    }
-
     /**
      * Renders a line area. <p>
      *
@@ -570,23 +599,34 @@ public abstract class AbstractRenderer
      */
     protected void renderLineArea(LineArea line) {
         List children = line.getInlineAreas();
-
+        int saveBP = currentBPPosition;
+        currentBPPosition += line.getSpaceBefore();
         for (int count = 0; count < children.size(); count++) {
             InlineArea inline = (InlineArea) children.get(count);
             renderInlineArea(inline);
         }
+        currentBPPosition = saveBP;
     }
 
+    /**
+     * Render the given InlineArea.
+     * @param inlineArea inline area text to render
+     */
     protected void renderInlineArea(InlineArea inlineArea) {
-        renderTextDecoration(inlineArea);
         if (inlineArea instanceof TextArea) {
             renderText((TextArea) inlineArea);
+        //} else if (inlineArea instanceof Character) {
+            //renderCharacter((Character) inlineArea);
+        } else if (inlineArea instanceof WordArea) {
+            renderWord((WordArea) inlineArea);
+        } else if (inlineArea instanceof SpaceArea) {
+            renderSpace((SpaceArea) inlineArea);
         } else if (inlineArea instanceof InlineParent) {
             renderInlineParent((InlineParent) inlineArea);
+        } else if (inlineArea instanceof InlineBlockParent) {
+            renderInlineBlockParent((InlineBlockParent) inlineArea);
         } else if (inlineArea instanceof Space) {
             renderInlineSpace((Space) inlineArea);
-        } else if (inlineArea instanceof Character) {
-            renderCharacter((Character) inlineArea);
         } else if (inlineArea instanceof Viewport) {
             renderViewport((Viewport) inlineArea);
         } else if (inlineArea instanceof Leader) {
@@ -594,14 +634,30 @@ public abstract class AbstractRenderer
         }
     }
 
-
-    /** @see org.apache.fop.render.Renderer */
+    /**
+     * Render the given Character.
+     * @param ch the character to render
+     * @deprecated Only TextArea should be used. This method will be removed eventually.
+     */
     protected void renderCharacter(Character ch) {
         currentIPPosition += ch.getAllocIPD();
     }
 
-    /** @see org.apache.fop.render.Renderer */
+    /** 
+     * Common method to render the background and borders for any inline area.
+     * The all borders and padding are drawn outside the specified area.
+     * @param area the inline area for which the background, border and padding is to be
+     * rendered
+     */
+    protected abstract void renderInlineAreaBackAndBorders(InlineArea area);
+    
+    /**
+     * Render the given Space.
+     * @param space the space to render
+     */
     protected void renderInlineSpace(Space space) {
+        space.setBPD(0);
+        renderInlineAreaBackAndBorders(space);
         // an inline space moves the inline progression position
         // for the current block by the width or height of the space
         // it may also have styling (only on this object) that needs
@@ -609,27 +665,80 @@ public abstract class AbstractRenderer
         currentIPPosition += space.getAllocIPD();
     }
 
-    /** @see org.apache.fop.render.Renderer */
+    /**
+     * Render the given Leader.
+     * @param area the leader to render
+     */
     protected void renderLeader(Leader area) {
         currentIPPosition += area.getAllocIPD();
     }
 
-    /** @see org.apache.fop.render.Renderer */
+    /**
+     * Render the given TextArea.
+     * @param text the text to render
+     */
     protected void renderText(TextArea text) {
-        currentIPPosition += text.getAllocIPD();
+        int saveIP = currentIPPosition;
+        int saveBP = currentBPPosition;
+        Iterator iter = text.getChildAreas().iterator();
+        while (iter.hasNext()) {
+            renderInlineArea((InlineArea) iter.next()); 
+        }
+        currentIPPosition = saveIP + text.getAllocIPD();
     }
 
-    /** @see org.apache.fop.render.Renderer */
+    /**
+     * Render the given WordArea.
+     * @param word the word to render
+     */
+    protected void renderWord(WordArea word) {
+        currentIPPosition += word.getAllocIPD();
+    }
+
+    /**
+     * Render the given SpaceArea.
+     * @param space the space to render
+     */
+    protected void renderSpace(SpaceArea space) {
+        currentIPPosition += space.getAllocIPD();
+    }
+
+    /**
+     * Render the given InlineParent.
+     * @param ip the inline parent to render
+     */
     protected void renderInlineParent(InlineParent ip) {
+        renderInlineAreaBackAndBorders(ip);
         int saveIP = currentIPPosition;
+        int saveBP = currentBPPosition;
+        currentIPPosition += ip.getBorderAndPaddingWidthStart();
+        currentBPPosition += ip.getOffset();
         Iterator iter = ip.getChildAreas().iterator();
         while (iter.hasNext()) {
             renderInlineArea((InlineArea) iter.next()); 
         }
         currentIPPosition = saveIP + ip.getAllocIPD();
+        currentBPPosition = saveBP;
     }
 
-    /** @see org.apache.fop.render.Renderer */
+    /**
+     * Render the given InlineBlockParent.
+     * @param ibp the inline block parent to render
+     */
+    protected void renderInlineBlockParent(InlineBlockParent ibp) {
+        renderInlineAreaBackAndBorders(ibp);
+        currentIPPosition += ibp.getBorderAndPaddingWidthStart();
+        // For inline content the BP position is updated by the enclosing line area
+        int saveBP = currentBPPosition;
+        currentBPPosition += ibp.getOffset();
+        renderBlock(ibp.getChildArea());
+        currentBPPosition = saveBP;
+    }
+
+    /**
+     * Render the given Viewport.
+     * @param viewport the viewport to render
+     */
     protected void renderViewport(Viewport viewport) {
         Area content = viewport.getContent();
         int saveBP = currentBPPosition;
@@ -684,32 +793,6 @@ public abstract class AbstractRenderer
         // Some renderers (ex. Text) don't support foreign objects.
     }
 
-    /**
-     * Set the default xml handler for the given mime type.
-     * @param mime MIME type
-     * @param handler XMLHandler to use
-     */
-    public void setDefaultXMLHandler(FOUserAgent foua, String mime,
-                                     XMLHandler handler) {
-        foua.defaults.put(mime, handler);
-    }
-
-    /**
-     * Add an xml handler for the given mime type and xml namespace.
-     * @param mime MIME type
-     * @param ns Namespace URI
-     * @param handler XMLHandler to use
-     */
-    public void addXMLHandler(FOUserAgent foua, String mime, String ns,
-                              XMLHandler handler) {
-        Map mh = (Map) foua.handlers.get(mime);
-        if (mh == null) {
-            mh = new java.util.HashMap();
-            foua.handlers.put(mime, mh);
-        }
-        mh.put(ns, handler);
-    }
-
     /**
      * Render the xml document with the given xml namespace.
      * The Render Context is by the handle to render into the current
@@ -718,29 +801,31 @@ public abstract class AbstractRenderer
      * @param doc DOM Document containing the source document
      * @param namespace Namespace URI of the document
      */
-    public void renderXML(FOUserAgent foua, RendererContext ctx, Document doc,
+    public void renderXML(RendererContext ctx, Document doc,
                           String namespace) {
-        String mime = ctx.getMimeType();
-        Map mh = (Map) foua.handlers.get(mime);
-        XMLHandler handler = null;
-        if (mh != null) {
-            handler = (XMLHandler) mh.get(namespace);
-        }
-        if (handler == null) {
-            handler = (XMLHandler) foua.defaults.get(mime);
-        }
+        XMLHandler handler = userAgent.getXMLHandlerRegistry().getXMLHandler(
+                this, namespace);
         if (handler != null) {
             try {
+                XMLHandlerConfigurator configurator
+                    = new XMLHandlerConfigurator(userAgent);
+                configurator.configure(ctx, namespace);
                 handler.handleXML(ctx, doc, namespace);
             } catch (Throwable t) {
                 // could not handle document
-                getLogger().error("Some XML content will be ignored. "
+                log.error("Some XML content will be ignored. "
                         + "Could not render XML", t);
             }
         } else {
-            // no handler found for document
-            getLogger().warn("Some XML content will be ignored. "
-                    + "No handler defined for XML: " + namespace);
+            if (warnedXMLHandlers == null) {
+                warnedXMLHandlers = new java.util.HashSet();
+            }
+            if (!warnedXMLHandlers.contains(namespace)) {
+                // no handler found for document
+                warnedXMLHandlers.add(namespace);
+                log.warn("Some XML content will be ignored. "
+                        + "No handler defined for XML: " + namespace);
+            }
         }
     }
 
@@ -753,4 +838,3 @@ public abstract class AbstractRenderer
         return null;
     }
 }
-
