@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.ext.awt.g2d;
 
 import java.awt.AlphaComposite;
@@ -93,7 +61,7 @@ import java.util.Map;
  * are implemented by <tt>AbstractGraphics2D</tt>
  *
  * @author <a href="mailto:vincent.hardy@eng.sun.com">Vincent Hardy</a>
- * @version $Id: AbstractGraphics2D.java,v 1.7 2003/08/08 11:39:04 vhardy Exp $
+ * @version $Id: AbstractGraphics2D.java 504084 2007-02-06 11:24:46Z dvholten $
  * @see org.apache.batik.ext.awt.g2d.GraphicContext
  */
 public abstract class AbstractGraphics2D extends Graphics2D implements Cloneable {
@@ -114,16 +82,15 @@ public abstract class AbstractGraphics2D extends Graphics2D implements Cloneable
      *        convertion. No text is output.
      *
      */
-    public AbstractGraphics2D(boolean textAsShapes){
+    public AbstractGraphics2D(boolean textAsShapes) {
         this.textAsShapes = textAsShapes;
     }
 
     /**
-     * This private constructor is used in create
-     *
-     * @see #create
+     * Creates a new AbstractGraphics2D from an existing instance.
+     * @param g the AbstractGraphics2D whose properties should be copied
      */
-    public AbstractGraphics2D(AbstractGraphics2D g){
+    public AbstractGraphics2D(AbstractGraphics2D g) {
         this.gc = (GraphicContext)g.gc.clone();
         this.gc.validateTransformStack();
         this.textAsShapes = g.textAsShapes;
@@ -273,8 +240,8 @@ public abstract class AbstractGraphics2D extends Graphics2D implements Cloneable
      * @return      a <code>Shape</code> object representing the
      *              current clipping area, or <code>null</code> if
      *              no clip is set.
-     * @see         java.awt.Graphics#getClipBounds
-     * @see         java.awt.Graphics#clipRect
+     * @see         java.awt.Graphics#getClipBounds()
+     * @see         java.awt.Graphics#clipRect(int, int, int, int)
      * @see         java.awt.Graphics#setClip(int, int, int, int)
      * @see         java.awt.Graphics#setClip(Shape)
      * @since       JDK1.1
@@ -562,7 +529,7 @@ public abstract class AbstractGraphics2D extends Graphics2D implements Cloneable
      * @see         java.awt.Graphics#drawPolygon(int[], int[], int)
      * @since       JDK1.1
      */
-    public void drawPolyline(int xPoints[], int yPoints[],
+    public void drawPolyline(int[] xPoints, int[] yPoints,
                              int nPoints){
         if(nPoints > 0){
             GeneralPath path = new GeneralPath();
@@ -590,10 +557,10 @@ public abstract class AbstractGraphics2D extends Graphics2D implements Cloneable
      * @param        xPoints   a an array of <code>x</code> coordinates.
      * @param        yPoints   a an array of <code>y</code> coordinates.
      * @param        nPoints   a the total number of points.
-     * @see          java.awt.Graphics#fillPolygon
+     * @see          java.awt.Graphics#fillPolygon(int[],int[],int)
      * @see          java.awt.Graphics#drawPolyline
      */
-    public void drawPolygon(int xPoints[], int yPoints[],
+    public void drawPolygon(int[] xPoints, int[] yPoints,
                             int nPoints){
         Polygon polygon = new Polygon(xPoints, yPoints, nPoints);
         draw(polygon);
@@ -620,7 +587,7 @@ public abstract class AbstractGraphics2D extends Graphics2D implements Cloneable
      * @param        nPoints   a the total number of points.
      * @see          java.awt.Graphics#drawPolygon(int[], int[], int)
      */
-    public void fillPolygon(int xPoints[], int yPoints[],
+    public void fillPolygon(int[] xPoints, int[] yPoints,
                             int nPoints){
         Polygon polygon = new Polygon(xPoints, yPoints, nPoints);
         fill(polygon);
@@ -899,7 +866,7 @@ public abstract class AbstractGraphics2D extends Graphics2D implements Cloneable
      * @see #setTransform
      * @see #setComposite
      * @see #clip
-     * @see #setClip
+     * @see #setClip(Shape)
      */
     public boolean drawImage(Image img,
                              AffineTransform xform,
@@ -913,7 +880,7 @@ public abstract class AbstractGraphics2D extends Graphics2D implements Cloneable
             }   catch(NoninvertibleTransformException e){
                                 // Should never happen since we checked the
                                 // matrix determinant
-                throw new Error();
+                throw new Error( e.getMessage() );
             }
 
             gc.transform(xform);
@@ -943,16 +910,15 @@ public abstract class AbstractGraphics2D extends Graphics2D implements Cloneable
      * img1 = op.filter(img, null);
      * drawImage(img1, new AffineTransform(1f,0f,0f,1f,x,y), null);
      * </pre>
-     * @param op the filter to be applied to the image before rendering
      * @param img the <code>BufferedImage</code> to be rendered
-     * @param x,&nbsp;y the location in user space where the upper left
-     * corner of the
-     * image is rendered
+     * @param op the filter to be applied to the image before rendering
+     * @param x the x coordinate in user space where the image is rendered
+     * @param y the y coordinate in user space where the image is rendered
      * @see #transform
      * @see #setTransform
      * @see #setComposite
      * @see #clip
-     * @see #setClip
+     * @see #setClip(Shape)
      */
     public void drawImage(BufferedImage img,
                           BufferedImageOp op,
@@ -977,16 +943,18 @@ public abstract class AbstractGraphics2D extends Graphics2D implements Cloneable
      * screen.
      *
      * @param g the <code>GlyphVector</code> to be rendered
-     * @param x,&nbsp;y the position in User Space where the glyphs should
-     * be rendered
+     * @param x the x position in user space where the glyphs should be
+     *        rendered
+     * @param y the y position in user space where the glyphs should be
+     *        rendered
      *
-     * @see java.awt.Font#createGlyphVector
+     * @see java.awt.Font#createGlyphVector(FontRenderContext, char[])
      * @see java.awt.font.GlyphVector
      * @see #setPaint
      * @see java.awt.Graphics#setColor
      * @see #setTransform
      * @see #setComposite
-     * @see #setClip
+     * @see #setClip(Shape)
      */
     public void drawGlyphVector(GlyphVector g, float x, float y){
         Shape glyphOutline = g.getOutline(x, y);
@@ -1015,12 +983,12 @@ public abstract class AbstractGraphics2D extends Graphics2D implements Cloneable
      * @return <code>true</code> if there is a hit; <code>false</code>
      * otherwise.
      * @see #setStroke
-     * @see #fill
-     * @see #draw
+     * @see #fill(Shape)
+     * @see #draw(Shape)
      * @see #transform
      * @see #setTransform
      * @see #clip
-     * @see #setClip
+     * @see #setClip(Shape)
      */
     public boolean hit(Rectangle rect,
                        Shape s,
@@ -1048,12 +1016,12 @@ public abstract class AbstractGraphics2D extends Graphics2D implements Cloneable
      * there is a security manager, its <code>checkPermission</code>
      * method is called with an <code>AWTPermission("readDisplayPixels")</code>
      * permission.
+     * @param comp the <code>Composite</code> object to be used for rendering
      * @throws SecurityException
      *         if a custom <code>Composite</code> object is being
      *         used to render to the screen and a security manager
      *         is set and its <code>checkPermission</code> method
      *         does not allow the operation.
-     * @param comp the <code>Composite</code> object to be used for rendering
      * @see java.awt.Graphics#setXORMode
      * @see java.awt.Graphics#setPaintMode
      * @see java.awt.AlphaComposite
@@ -1072,10 +1040,8 @@ public abstract class AbstractGraphics2D extends Graphics2D implements Cloneable
      * @param paint the <code>Paint</code> object to be used to generate
      * color during the rendering process, or <code>null</code>
      * @see java.awt.Graphics#setColor
-     * @see GradientPaint
-     * @see TexturePaint
      */
-    public void setPaint( Paint paint ){
+    public void setPaint(Paint paint) {
         gc.setPaint(paint);
     }
 
@@ -1084,7 +1050,6 @@ public abstract class AbstractGraphics2D extends Graphics2D implements Cloneable
      * Sets the <code>Stroke</code> for the <code>Graphics2D</code> context.
      * @param s the <code>Stroke</code> object to be used to stroke a
      * <code>Shape</code> during the rendering process
-     * @see BasicStroke
      */
     public void setStroke(Stroke s){
         gc.setStroke(s);
@@ -1235,7 +1200,8 @@ public abstract class AbstractGraphics2D extends Graphics2D implements Cloneable
      * Rotating with a positive angle theta rotates points on the positive
      * x axis toward the positive y axis.
      * @param theta the angle of rotation in radians
-     * @param x,&nbsp;y coordinates of the origin of the rotation
+     * @param x the x coordinate of the origin of the rotation
+     * @param y the y coordinate of the origin of the rotation
      */
     public void rotate(double theta, double x, double y){
         gc.rotate(theta, x, y);
@@ -1415,9 +1381,9 @@ public abstract class AbstractGraphics2D extends Graphics2D implements Cloneable
      * The <i>user clip</i> modified by this method is independent of the
      * clipping associated with device bounds and visibility.  If no clip has
      * previously been set, or if the clip has been cleared using
-     * {@link Graphics#setClip(Shape) setClip} with a <code>null</code>
-     * argument, the specified <code>Shape</code> becomes the new
-     * user clip.
+     * {@link java.awt.Graphics#setClip(Shape) setClip} with a
+     * <code>null</code> argument, the specified <code>Shape</code> becomes
+     * the new user clip.
      * @param s the <code>Shape</code> to be intersected with the current
      *          <code>Clip</code>.  If <code>s</code> is <code>null</code>,
      *          this method clears the current <code>Clip</code>.
@@ -1444,11 +1410,18 @@ public abstract class AbstractGraphics2D extends Graphics2D implements Cloneable
      *
      * @return a reference to an instance of FontRenderContext.
      * @see java.awt.font.FontRenderContext
-     * @see java.awt.Font#createGlyphVector
+     * @see java.awt.Font#createGlyphVector(FontRenderContext,char[])
      * @see java.awt.font.TextLayout
      * @since     JDK1.2
      */
     public FontRenderContext getFontRenderContext(){
         return gc.getFontRenderContext();
     }
+
+    /**
+     * @return the {@link GraphicContext} of this <code>Graphics2D</code>.
+     */
+    public GraphicContext getGraphicContext() {
+        return gc;
+    }
 }
