@@ -1,60 +1,27 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.ext.awt.geom;
 
 import java.awt.Rectangle;
 import java.io.Serializable;
 import java.util.Arrays;
 import java.util.Collection;
-import java.util.List;
 import java.util.Comparator;
 import java.util.Iterator;
 import java.util.ListIterator;
@@ -69,14 +36,24 @@ import java.util.NoSuchElementException;
  * this reason it uses Rectangle not Rectangle2D).
  *
  * @author <a href="mailto:deweese@apache.org">Thomas DeWeese</a>
- * @version $Id: RectListManager.java,v 1.8 2003/11/07 18:25:03 deweese Exp $ */
-
+ * @version $Id: RectListManager.java 501844 2007-01-31 13:54:05Z dvholten $
+ */
 public class RectListManager implements Collection {
     Rectangle [] rects = null;
     int size = 0;
 
     Rectangle bounds = null;
 
+    public void dump() {
+        System.err.println("RLM: " + this + " Sz: " + size);
+        System.err.println("Bounds: " + getBounds());
+        for (int i=0; i<size; i++) {
+            Rectangle r = rects[i];
+            System.err.println("  [" + r.x + ", " + r.y + ", " +
+                               r.width + ", " + r.height + ']' );
+        }
+    }
+
     /**
      * The comparator used to sort the elements of this List.
      * Sorts on x value of Rectangle.
@@ -91,15 +68,16 @@ public class RectListManager implements Collection {
         this.rects = new Rectangle[rects.size()];
         Iterator i = rects.iterator();
         int j=0;
-        while (i.hasNext())
+        while (i.hasNext())          // todo can be replaced by rects.toArray()
             this.rects[j++] = (Rectangle)i.next();
         this.size  = this.rects.length;
 
+
         Arrays.sort(this.rects, comparator);
     }
 
     /**
-     * Construct a <tt>RectListManager</tt> from an Array of 
+     * Construct a <tt>RectListManager</tt> from an Array of
      * <tt>Rectangles</tt>
      * @param rects Array of <tt>Rectangles</tt>, must not contain
      *              any null entries.
@@ -109,7 +87,7 @@ public class RectListManager implements Collection {
     }
 
     /**
-     * Construct a <tt>RectListManager</tt> from an Array of 
+     * Construct a <tt>RectListManager</tt> from an Array of
      * <tt>Rectangles</tt>
      * @param rects Array of <tt>Rectangles</tt>, must not contain
      *              any null entries in the range [off, off+sz-1].
@@ -124,7 +102,7 @@ public class RectListManager implements Collection {
     }
 
     /**
-     * Construct a <tt>RectListManager</tt> from another 
+     * Construct a <tt>RectListManager</tt> from another
      * <tt>RectListManager</tt> (data is copied).
      * @param rlm RectListManager to copy.
      */
@@ -141,7 +119,7 @@ public class RectListManager implements Collection {
         add(rect);
     }
 
-    
+
     /**
      * Construct an initially empty <tt>RectListManager</tt>.
      */
@@ -161,7 +139,7 @@ public class RectListManager implements Collection {
     }
 
     public Rectangle getBounds() {
-        if (bounds != null ) 
+        if (bounds != null )
             return bounds;
         if (size == 0) return null;
         bounds = new Rectangle(rects[0]);
@@ -175,9 +153,9 @@ public class RectListManager implements Collection {
                 bounds.height = bounds.y+bounds.height-r.y;
                 bounds.y = r.y;
             }
-            if (r.x+r.width > bounds.x+bounds.width) 
+            if (r.x+r.width > bounds.x+bounds.width)
                 bounds.width = r.x+r.width-bounds.x;
-            if (r.y+r.height > bounds.y+bounds.height) 
+            if (r.y+r.height > bounds.y+bounds.height)
                 bounds.height = r.y+r.height-bounds.y;
         }
         return bounds;
@@ -186,7 +164,7 @@ public class RectListManager implements Collection {
     /**
      * Standard <tt>Object</tt> clone method.
      */
-    public Object clone() {
+    public Object clone() throws CloneNotSupportedException {
         return copy();
     }
 
@@ -207,11 +185,11 @@ public class RectListManager implements Collection {
      * Returns true if this collection contains no elements.
      */
     public boolean isEmpty() { return (size==0); }
-    
+
     public void clear() {
-        for (int i=0; i<size; i++) 
-            rects[i] = null;
+        Arrays.fill( rects, null );
         size=0;
+        bounds = null;
     }
 
     /**
@@ -222,7 +200,7 @@ public class RectListManager implements Collection {
     }
 
     /**
-     * Returns a list iterator of the elements in this list 
+     * Returns a list iterator of the elements in this list
      * (in proper sequence).
      */
     public ListIterator listIterator() {
@@ -235,21 +213,28 @@ public class RectListManager implements Collection {
         return ret;
     }
 
-    public Object [] toArray(Object []a) {
+    /**
+     * fill the given array a with values from my internal <code>rects</code>.
+     * when a is not large enough, a new array is allocated, filled and returned.
+     * the method works only, when a is a Object[] or a Rectange[].
+     * When this is not the case, the a[] is just cleared.
+     *
+     * @param a array to fill (must not be null!)
+     * @return the content of rects, either in a[] or a fresh array.
+     */
+    public Object [] toArray(Object[] a) {
         Class t = a.getClass().getComponentType();
-        if ((t != Object.class) &
+        if ((t != Object.class) &&
             (t != Rectangle.class)) {
             // Nothing here for it...
-            for (int i=0; i<a.length; i++)
-                a[i] = null;
+            Arrays.fill( a, null );
             return a;
         }
-        
+
         if (a.length < size)
             a = new Rectangle[size];
         System.arraycopy(rects, 0, a, 0, size);
-        for (int i=size; i<a.length; i++)
-            a[i] = null;
+        Arrays.fill( a, size, a.length, null );
 
         return a;
     }
@@ -287,7 +272,7 @@ public class RectListManager implements Collection {
                 // All 'null' from center to r so skip them
                 r = (l+r)/2;
                 idx = (l+r)/2;
-                if (l>r) 
+                if (l>r)
                     idx=l;
                 while ((rects[idx] == null) && (idx > l)) idx--;
                 if (rects[idx] == null) {
@@ -298,19 +283,19 @@ public class RectListManager implements Collection {
             if (rect.x == rects[idx].x) break;
             if (rect.x <  rects[idx].x) {
                 if (idx == 0) break;
-                if ((rects[idx-1] != null) && 
+                if ((rects[idx-1] != null) &&
                     (rect.x >= rects[idx-1].x)) break;
                 r = idx-1;
             } else {
                 if (idx == size-1)  {idx++; break; }
-                if ((rects[idx+1] != null) && 
+                if ((rects[idx+1] != null) &&
                     (rect.x <= rects[idx+1].x)) { idx++; break;}
                 l = idx+1;
             }
         }
 
         if (idx < size) {
-            System.arraycopy(rects, idx, 
+            System.arraycopy(rects, idx,
                              rects, idx+1, size-idx);
         }
 
@@ -323,6 +308,7 @@ public class RectListManager implements Collection {
 
         rects[idx] = rect;
         size++;
+        bounds=null;
     }
 
     public boolean addAll(Collection c) {
@@ -339,7 +325,7 @@ public class RectListManager implements Collection {
         Rectangle rect = (Rectangle)o;
         int l=0, r=size-1, idx=0;
         while (l <= r) {
-            idx = (l+r)/2;
+            idx = (l+r) >>> 1;
             if (rect.x == rects[idx].x) break;
             if (rect.x <  rects[idx].x) {
                 if (idx == 0) break;
@@ -370,7 +356,7 @@ public class RectListManager implements Collection {
         return false;
     }
 
-    /** 
+    /**
      * Returns true if this collection contains all of the elements in
      * the specified collection.
      */
@@ -417,7 +403,7 @@ public class RectListManager implements Collection {
     public boolean remove(Rectangle rect) {
         int l=0, r=size-1, idx=0;
         while (l <= r) {
-            idx = (l+r)/2;
+            idx = (l+r) >>> 1;
             if (rect.x == rects[idx].x) break;
             if (rect.x <  rects[idx].x) {
                 if (idx == 0) break;
@@ -437,6 +423,7 @@ public class RectListManager implements Collection {
             if (rects[idx].equals(rect)) {
                 System.arraycopy(rects, idx+1, rects, idx, size-idx);
                 size--;
+                bounds = null;
                 return true;
             }
             if (rects[idx].x != rect.x)  break;
@@ -447,6 +434,7 @@ public class RectListManager implements Collection {
             if (rects[idx].equals(rect)) {
                 System.arraycopy(rects, idx+1, rects, idx, size-idx);
                 size--;
+                bounds = null;
                 return true;
             }
             if (rects[idx].x != rect.x)  break;
@@ -467,7 +455,7 @@ public class RectListManager implements Collection {
         boolean ret = false;
         for (int j=0, i=0; j<rlm.size; j++) {
             i=xChange;
-            while ((rects[i] == null) || 
+            while ((rects[i] == null) ||
                    (rects[i].x < rlm.rects[j].x)) {
                 i++;
                 if (i == size) break;
@@ -497,11 +485,12 @@ public class RectListManager implements Collection {
         if (ret) {
             int j=0, i=0;
             while (i<size) {
-                if (rects[i] != null) 
+                if (rects[i] != null)
                     rects[j++] = rects[i];
                 i++;
             }
             size = j;
+            bounds = null;
         }
         return ret;
     }
@@ -550,11 +539,12 @@ public class RectListManager implements Collection {
         if (ret) {
             int j=0, i=0;
             while (i<size) {
-                if (rects[i] != null) 
+                if (rects[i] != null)
                     rects[j++] = rects[i];
                 i++;
             }
             size = j;
+            bounds = null;
         }
         return ret;
     }
@@ -574,13 +564,14 @@ public class RectListManager implements Collection {
         if (rects.length < (size+rlm.size)) {
             dst = new Rectangle[size+rlm.size];
         }
-        
+
         if (size == 0) {
             System.arraycopy(rlm.rects, 0, dst, size, rlm.size);
             size = rlm.size;
+            bounds = null;
             return;
         }
-        
+
         Rectangle [] src1   = rlm.rects;
         int          src1Sz = rlm.size;
         int          src1I  = src1Sz-1;
@@ -592,10 +583,10 @@ public class RectListManager implements Collection {
         int dstI = size+rlm.size-1;
         int x1 = src1[src1I].x;
         int x2 = src2[src2I].x;
-        
+
         while (dstI >= 0) {
             if (x1 <= x2) {
-                dst[dstI] = src2[src2I]; 
+                dst[dstI] = src2[src2I];
                 if (src2I == 0) {
                     System.arraycopy(src1, 0, dst, 0, src1I+1);
                     break;
@@ -603,7 +594,7 @@ public class RectListManager implements Collection {
                 src2I--;
                 x2 = src2[src2I].x;
             } else {
-                dst[dstI] = src1[src1I]; 
+                dst[dstI] = src1[src1I];
                 if (src1I == 0) {
                     System.arraycopy(src2, 0, dst, 0, src2I+1);
                     break;
@@ -615,6 +606,7 @@ public class RectListManager implements Collection {
         }
         rects = dst;
         size += rlm.size;
+        bounds = null;
     }
 
     public void mergeRects(int overhead, int lineOverhead) {
@@ -626,7 +618,7 @@ public class RectListManager implements Collection {
         for (int j, i=0; i<size; i++) {
             r = rects[i];
             if (r == null) continue;
-            cost1 = (overhead                 + 
+            cost1 = (overhead                 +
                      (r.height*lineOverhead) +
                      (r.height*r.width));
             do {
@@ -639,12 +631,12 @@ public class RectListManager implements Collection {
                         j = size;
                         break;
                     }
-                    cost2 = (overhead                 + 
+                    cost2 = (overhead                 +
                              (cr.height*lineOverhead) +
                              (cr.height*cr.width));
 
                     mr = r.union(cr);
-                    cost3 = (overhead                 + 
+                    cost3 = (overhead                 +
                              (mr.height*lineOverhead) +
                              (mr.height*mr.width));
                     if (cost3 <= cost1+cost2) {
@@ -666,7 +658,7 @@ public class RectListManager implements Collection {
                             // Collapse null entries in first three
                             // (That share common 'x').
                             if (k<3) splits[l++] = sr;
-                            splitCost += (overhead                 + 
+                            splitCost += (overhead                 +
                                           (sr.height*lineOverhead) +
                                           (sr.height*sr.width));
                         }
@@ -708,6 +700,7 @@ public class RectListManager implements Collection {
             i++;
         }
         size = j;
+        bounds=null;
         r = getBounds();
         if (r == null) return;
         if (overhead + (r.height*lineOverhead) + (r.height*r.width) < area) {
@@ -724,14 +717,14 @@ public class RectListManager implements Collection {
 
         for(int i=0; i<size; i++) {
             r = rects[i]; // Canidate rect...
-            cost = (overhead                + 
+            cost = (overhead                +
                     (r.height*lineOverhead) +
                     (r.height*r.width));
             for (int j=jMin; j<rlm.size; j++) {
                 sr = rlm.rects[j]; // subtraction rect.
 
                 // Check if the canidate rect starts after
-                // the end of this rect in 'x' if so 
+                // the end of this rect in 'x' if so
                 // go to the next one.
                 if (sr.x+sr.width < r.x) {
                     // If this was jMin then increment jMin (no
@@ -759,8 +752,8 @@ public class RectListManager implements Collection {
                 Rectangle tmpR;
                 for (int k=0; k<4; k++) {
                     tmpR = splits[k];
-                    if (tmpR != null) 
-                        splitCost += (overhead                   + 
+                    if (tmpR != null)
+                        splitCost += (overhead                   +
                                       (tmpR.height*lineOverhead) +
                                       (tmpR.height*tmpR.width));
                 }
@@ -773,7 +766,7 @@ public class RectListManager implements Collection {
                     // subtract list has been merged as this will help
                     // reduce the instances where this will happen.
                     continue;
-                
+
                 // Collapse null entries in first three elements
                 // split 0, 1, 2 (entries that share a common 'x').
                 int l = 0;
@@ -797,9 +790,9 @@ public class RectListManager implements Collection {
                 // the split, since it only shrunk it didn't grow,
                 // we know that the previous subtract rects don't
                 // intersect it.
-                r        = splits[0]; 
+                r        = splits[0];
                 rects[i] = r;
-                cost = (overhead                + 
+                cost = (overhead                +
                         (r.height*lineOverhead) +
                         (r.height*r.width));
 
@@ -821,11 +814,12 @@ public class RectListManager implements Collection {
         while (i<size) {
             if (rects[i].width == 0)
                 rects[i] = null;
-            else 
+            else
                 rects[j++] = rects[i];
             i++;
         }
         size = j;
+        bounds = null;
     }
 
     protected void splitRect(Rectangle r, Rectangle sr,
@@ -875,13 +869,13 @@ public class RectListManager implements Collection {
         } else {
             splits[1] = null;
         }
-                
+
         if ((rx0 < srx0) && (rx1 >= srx0)) {
             splits[2] = new Rectangle(rx0, ry0, srx0-rx0, ry1-ry0+1);
         } else {
             splits[2] = null;
         }
-                
+
         if ((rx0 <= srx1) && (rx1 > srx1)) {
             splits[3]= new Rectangle(srx1+1, ry0, rx1-srx1, ry1-ry0+1);
         } else {
@@ -889,7 +883,7 @@ public class RectListManager implements Collection {
         }
     }
 
-    protected void insertRects(Rectangle[] rects, int srcPos, 
+    protected void insertRects(Rectangle[] rects, int srcPos,
                                int dstPos, int len) {
         if (len == 0) return;
 
@@ -901,19 +895,18 @@ public class RectListManager implements Collection {
             this.rects[i+len] = this.rects[i];
 
         // Put the new rects in.
-        for (int i=0; i<len; i++)
-            this.rects[i+dstPos] = rects[i+srcPos];
+        System.arraycopy( rects, srcPos, this.rects, dstPos, len );
 
         size += len;
     }
 
     public void ensureCapacity(int sz) {
-        if (sz <= rects.length) 
+        if (sz <= rects.length)
             return;
         int nSz = rects.length + (rects.length>>1) + 1;
         while (nSz < sz)
             nSz+=(nSz>>1)+1;
-        
+
         Rectangle [] nRects = new Rectangle[nSz];
         System.arraycopy(rects, 0, nRects, 0, size);
 
@@ -945,7 +938,7 @@ public class RectListManager implements Collection {
         public boolean hasNext() { return idx < size; }
         public int nextIndex() { return idx; }
         public Object next() {
-            if (idx >= size) 
+            if (idx >= size)
                 throw new NoSuchElementException("No Next Element");
             forward = true;
             removeOk = true;
@@ -955,13 +948,13 @@ public class RectListManager implements Collection {
         public boolean hasPrevious() { return idx > 0; }
         public int previousIndex() { return idx-1; }
         public Object previous() {
-            if (idx <= 0) 
+            if (idx <= 0)
                 throw new NoSuchElementException("No Previous Element");
             forward = false;
             removeOk = true;
             return rects[--idx];
         }
-            
+
         public void remove() {
             if (!removeOk)
                 throw new IllegalStateException
@@ -1013,7 +1006,7 @@ public class RectListManager implements Collection {
                         ("RectListManager entries must be sorted");
             }
             ensureCapacity(size+1);
-            if (idx != size) 
+            if (idx != size)
                 System.arraycopy(rects, idx, rects, idx+1, size-idx);
             rects[idx] = r;
             idx++;
