@@ -1,57 +1,26 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.swing;
 
 import java.awt.Dimension;
 import java.awt.EventQueue;
+// import java.awt.Rectangle;
 import java.awt.event.ActionEvent;
 import java.awt.event.InputEvent;
 import java.awt.event.KeyEvent;
@@ -61,7 +30,10 @@ import java.awt.event.MouseMotionAdapter;
 import java.awt.geom.AffineTransform;
 import java.beans.PropertyChangeListener;
 import java.beans.PropertyChangeSupport;
+import java.util.Iterator;
 import java.util.List;
+import java.util.Map;
+import java.util.WeakHashMap;
 
 import javax.swing.AbstractAction;
 import javax.swing.ActionMap;
@@ -73,21 +45,26 @@ import javax.swing.KeyStroke;
 import javax.swing.ToolTipManager;
 
 import org.apache.batik.bridge.UserAgent;
-import org.apache.batik.dom.svg.ExtensibleSVGDOMImplementation;
-import org.apache.batik.dom.svg.SVGDOMImplementation;
+import org.apache.batik.css.engine.CSSEngine;
+import org.apache.batik.dom.events.NodeEventTarget;
 import org.apache.batik.swing.gvt.AbstractImageZoomInteractor;
 import org.apache.batik.swing.gvt.AbstractPanInteractor;
 import org.apache.batik.swing.gvt.AbstractResetTransformInteractor;
 import org.apache.batik.swing.gvt.AbstractRotateInteractor;
 import org.apache.batik.swing.gvt.AbstractZoomInteractor;
 import org.apache.batik.swing.gvt.Interactor;
+// import org.apache.batik.swing.gvt.Overlay;
 import org.apache.batik.swing.svg.JSVGComponent;
 import org.apache.batik.swing.svg.SVGDocumentLoaderEvent;
 import org.apache.batik.swing.svg.SVGUserAgent;
 import org.apache.batik.util.SVGConstants;
 import org.apache.batik.util.XMLConstants;
+// import org.apache.batik.util.gui.DOMViewer;
+// import org.apache.batik.util.gui.DOMViewerController;
+// import org.apache.batik.util.gui.ElementOverlayManager;
 import org.apache.batik.util.gui.JErrorPane;
-import org.w3c.dom.DOMImplementation;
+
+// import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.w3c.dom.Node;
 import org.w3c.dom.events.Event;
@@ -105,7 +82,7 @@ import org.w3c.dom.svg.SVGDocument;
  *
  * @author <a href="mailto:tkormann@apache.org">Thierry Kormann</a>
  * @author <a href="mailto:stephane@hillion.org">Stephane Hillion</a>
- * @version $Id: JSVGCanvas.java,v 1.44 2003/09/07 22:25:28 deweese Exp $
+ * @version $Id: JSVGCanvas.java 594379 2007-11-13 01:08:28Z cam $
  */
 public class JSVGCanvas extends JSVGComponent {
 
@@ -209,21 +186,47 @@ public class JSVGCanvas extends JSVGComponent {
      * Keeps track of the last known mouse position over the canvas.
      * This is used for displaying tooltips at the right location.
      */
-    protected LocationListener locationListener = null;
+    protected LocationListener locationListener = new LocationListener();
+
+    /**
+     * Mapping of elements to listeners so they can be removed,
+     * if the tooltip is removed.
+     */
+    protected Map toolTipMap = null;
+    protected EventListener toolTipListener = new ToolTipModifier();
+    protected EventTarget   lastTarget = null;
+    protected Map toolTipDocs = null;
+    /**
+     * This is used as the value in the toolTipDocs WeakHashMap.
+     * This way we can tell if a document has already been added.
+     */
+    protected static final Object MAP_TOKEN = new Object();
+    /**
+     * The time of the last tool tip event.
+     */
+    protected long lastToolTipEventTimeStamp;
+
+    /**
+     * The target for which the last tool tip event was fired.
+     */
+    protected EventTarget lastToolTipEventTarget;
+
+
 
     /**
      * Creates a new JSVGCanvas.
      */
     public JSVGCanvas() {
         this(null, true, true);
+        addMouseMotionListener(locationListener);
     }
 
     /**
      * Creates a new JSVGCanvas.
      *
      * @param ua a SVGUserAgent instance or null.
-     * @param eventEnabled Whether the GVT tree should be reactive to mouse and
-     * key events.
+     * @param eventsEnabled Whether the GVT tree should be reactive to mouse
+     *                      and key events.
      * @param selectableText Whether the text should be selectable.
      */
     public JSVGCanvas(SVGUserAgent ua,
@@ -253,6 +256,7 @@ public class JSVGCanvas extends JSVGComponent {
 
             installKeyboardActions();
         }
+        addMouseMotionListener(locationListener);
     }
 
     /**
@@ -544,7 +548,7 @@ public class JSVGCanvas extends JSVGComponent {
     /**
      * To hide the listener methods. This class just reset the tooltip.
      */
-    protected class CanvasSVGListener extends JSVGComponent.SVGListener {
+    protected class CanvasSVGListener extends ExtendedSVGListener {
 
         /**
          * Called when the loading of a document was started.
@@ -556,6 +560,82 @@ public class JSVGCanvas extends JSVGComponent {
 
     }
 
+    protected void installSVGDocument(SVGDocument doc) {
+        if (toolTipDocs != null) {
+            Iterator i = toolTipDocs.keySet().iterator();
+            while (i.hasNext()) {
+                SVGDocument ttdoc;
+                ttdoc = (SVGDocument)i.next();
+                if (ttdoc == null) continue;
+
+                NodeEventTarget root;
+                root = (NodeEventTarget)ttdoc.getRootElement();
+                if (root == null) continue;
+                root.removeEventListenerNS
+                    (XMLConstants.XML_EVENTS_NAMESPACE_URI,
+                     SVGConstants.SVG_EVENT_MOUSEOVER,
+                     toolTipListener, false);
+                root.removeEventListenerNS
+                    (XMLConstants.XML_EVENTS_NAMESPACE_URI,
+                     SVGConstants.SVG_EVENT_MOUSEOUT,
+                     toolTipListener, false);
+            }
+            toolTipDocs = null;
+        }
+        lastTarget = null;
+
+        if (toolTipMap != null) {
+            toolTipMap.clear();
+        }
+
+        super.installSVGDocument(doc);
+    }
+
+//     // DOMViewerController
+// 
+//     /**
+//      * DOMViewerController implementation.
+//      */
+//     protected class CanvasDOMViewerController implements DOMViewerController {
+// 
+//         public boolean canEdit() {
+//             return getUpdateManager() != null;
+//         }
+// 
+//         public ElementOverlayManager createSelectionManager() {
+//             if (canEdit()) {
+//                 return new ElementOverlayManager(JSVGCanvas.this);
+//             }
+//             return null;
+//         }
+// 
+//         public Document getDocument() {
+//             return svgDocument;
+//         }
+// 
+//         public void performUpdate(Runnable r) {
+//             if (canEdit()) {
+//                 getUpdateManager().getUpdateRunnableQueue().invokeLater(r);
+//             } else {
+//                 r.run();
+//             }
+//         }
+// 
+//         public void removeSelectionOverlay(Overlay selectionOverlay) {
+//             getOverlays().remove(selectionOverlay);
+//         }
+// 
+//         public void selectNode(Node node) {
+//             DOMViewer domViewer = new DOMViewer(this);
+//             Rectangle fr = getBounds();
+//             Dimension td = domViewer.getSize();
+//             domViewer.setLocation(fr.x + (fr.width - td.width) / 2,
+//                                   fr.y + (fr.height - td.height) / 2);
+//             domViewer.setVisible(true);
+//             domViewer.selectNode(node);
+//         }
+//     }
+
     // ----------------------------------------------------------------------
     // Actions
     // ----------------------------------------------------------------------
@@ -769,7 +849,7 @@ public class JSVGCanvas extends JSVGComponent {
 
     /**
      * The <tt>CanvasUserAgent</tt> only adds tooltips to the behavior of the
-     * default <tt>BridgeUserAgent</tt>.<br /> A tooltip will be displayed
+     * default <tt>BridgeUserAgent</tt>. A tooltip will be displayed
      * wheneven the mouse lingers over an element which has a &lt;title&gt; or a
      * &lt;desc&gt; child element.
      */
@@ -807,90 +887,136 @@ public class JSVGCanvas extends JSVGComponent {
 
             // Don't handle tool tips unless we are interactive.
             if (!isInteractive()) return;
-            
+
             if (!SVGConstants.SVG_NAMESPACE_URI.equals(elt.getNamespaceURI()))
                 return;
 
+            // Don't handle tool tips for the root SVG element.
+            if (elt.getParentNode() ==
+                elt.getOwnerDocument().getDocumentElement()) {
+                return;
+            }
+
+            Element parent;
+            // When node is removed data is old parent node
+            // since we can't get it otherwise.
+            if (data instanceof Element) parent = (Element)data;
+            else                         parent = (Element)elt.getParentNode();
+
+            Element descPeer = null;
+            Element titlePeer = null;
             if (elt.getLocalName().equals(SVGConstants.SVG_TITLE_TAG)) {
-                // If there is a <desc> peer, do nothing as the tooltip will
-                // be handled when handleElement is invoked for the <desc>
-                // peer.
-                if (hasPeerWithTag
-                    (elt,
-                     SVGConstants.SVG_NAMESPACE_URI,
-                     SVGConstants.SVG_DESC_TAG)){
-                    return;
-                }
-                
-                elt.normalize();
-                if (elt.getFirstChild() == null) {
-                    return;
-                }
-                String toolTip = elt.getFirstChild().getNodeValue();
-                if (toolTip == null || toolTip.length() == 0) {
-                    return;
-                }
-                toolTip = Messages.formatMessage
-                    (TOOLTIP_TITLE_ONLY,
-                     new Object[]{toFormattedHTML(toolTip)});
-                
-                setToolTip((Element)(elt.getParentNode()), toolTip);
-            } else if (elt.getLocalName().equals
-                       (SVGConstants.SVG_DESC_TAG)) {
-                //  If there is a <title> peer, prepend its content to the
-                // content of the <desc> element.
-                elt.normalize();
-                if (elt.getFirstChild() == null) {
-                    return;
-                }
-                String toolTip = elt.getFirstChild().getNodeValue();
-                if (toolTip == null || toolTip.length() == 0) {
-                    return;
-                }
-                
-                Element titlePeer =
-                    getPeerWithTag(elt,
-                                   SVGConstants.SVG_NAMESPACE_URI,
-                                   SVGConstants.SVG_TITLE_TAG);
-                if (titlePeer != null) {
-                    titlePeer.normalize();
-                    toolTip = Messages.formatMessage(TOOLTIP_TITLE_AND_TEXT,
-                                                     new Object[] {
-                                                         toFormattedHTML(titlePeer.getFirstChild().getNodeValue()),
-                                                         toFormattedHTML(toolTip)});
+                if (data == Boolean.TRUE)
+                    titlePeer = elt;
+                descPeer = getPeerWithTag(parent,
+                                           SVGConstants.SVG_NAMESPACE_URI,
+                                           SVGConstants.SVG_DESC_TAG);
+            } else if (elt.getLocalName().equals(SVGConstants.SVG_DESC_TAG)) {
+                if (data == Boolean.TRUE)
+                    descPeer = elt;
+                titlePeer = getPeerWithTag(parent,
+                                           SVGConstants.SVG_NAMESPACE_URI,
+                                           SVGConstants.SVG_TITLE_TAG);
+            }
+
+            String titleTip = null;
+            if (titlePeer != null) {
+                titlePeer.normalize();
+                if (titlePeer.getFirstChild() != null)
+                    titleTip = titlePeer.getFirstChild().getNodeValue();
+            }
+
+            String descTip = null;
+            if (descPeer != null) {
+                descPeer.normalize();
+                if (descPeer.getFirstChild() != null)
+                    descTip = descPeer.getFirstChild().getNodeValue();
+            }
+
+            final String toolTip;
+            if ((titleTip != null) && (titleTip.length() != 0)) {
+                if ((descTip != null) && (descTip.length() != 0)) {
+                    toolTip = Messages.formatMessage
+                        (TOOLTIP_TITLE_AND_TEXT,
+                         new Object[] { toFormattedHTML(titleTip),
+                                        toFormattedHTML(descTip)});
                 } else {
-                    toolTip =
-                        Messages.formatMessage
+                    toolTip = Messages.formatMessage
+                        (TOOLTIP_TITLE_ONLY,
+                         new Object[]{toFormattedHTML(titleTip)});
+                }
+            } else {
+                if ((descTip != null) && (descTip.length() != 0)) {
+                    toolTip = Messages.formatMessage
                         (TOOLTIP_DESC_ONLY,
-                         new Object[]{toFormattedHTML(toolTip)});
+                         new Object[]{toFormattedHTML(descTip)});
+                } else {
+                    toolTip = null;
+                }
+            }
+
+            if (toolTip == null) {
+                removeToolTip(parent);
+                return;
+            }
+
+            if (lastTarget != parent) {
+                setToolTip(parent, toolTip);
+            } else {
+                // Already has focus check if it already has tip text.
+                Object o = null;
+                if (toolTipMap != null) {
+                    o = toolTipMap.get(parent);
+                    toolTipMap.put(parent, toolTip);
+                }
+
+                if (o != null) {
+                    // Update components tooltip text now.
+                    EventQueue.invokeLater(new Runnable() {
+                            public void run() {
+                                setToolTipText(toolTip);
+                                MouseEvent e = new MouseEvent
+                                    (JSVGCanvas.this,
+                                     MouseEvent.MOUSE_MOVED,
+                                     System.currentTimeMillis(),
+                                     0,
+                                     locationListener.getLastX(),
+                                     locationListener.getLastY(),
+                                     0,
+                                     false);
+                                ToolTipManager.sharedInstance().mouseMoved(e);
+                            }
+                        });
+                } else {
+                    EventQueue.invokeLater(new ToolTipRunnable(toolTip));
                 }
-                
-                setToolTip((Element)(elt.getParentNode()), toolTip);
             }
         }
 
         /**
          * Converts line breaks to HTML breaks and encodes special entities.
-         * Poor way of replacing '<', '>', '"', '&' and ''' in attribute values.
+         * Poor way of replacing '<', '>' and '&' in content.
          */
         public String toFormattedHTML(String str) {
             StringBuffer sb = new StringBuffer(str);
-            replace(sb, XML_CHAR_AMP, XML_ENTITY_AMP);
+            replace(sb, XML_CHAR_AMP, XML_ENTITY_AMP);  // Must go first!
             replace(sb, XML_CHAR_LT, XML_ENTITY_LT);
             replace(sb, XML_CHAR_GT, XML_ENTITY_GT);
             replace(sb, XML_CHAR_QUOT, XML_ENTITY_QUOT);
-            replace(sb, XML_CHAR_APOS, XML_ENTITY_APOS);
+            // Dont' quote "'" apostrphe since the display doesn't
+            // seem to understand it.
+            // replace(sb, XML_CHAR_APOS, XML_ENTITY_APOS);
             replace(sb, '\n', "<br>");
             return sb.toString();
         }
 
-        protected void replace(StringBuffer s, char c, String r) {
-            String v = s.toString() + 1;
+        protected void replace(StringBuffer sb, char c, String r) {
+            String v = sb.toString();
             int i = v.length();
 
-            while( (i=v.lastIndexOf(c, --i)) != -1 ) {
-                s.deleteCharAt(i);
-                s.insert(i, r);
+            while( (i=v.lastIndexOf(c, i-1)) != -1 ) {
+                sb.deleteCharAt(i);
+                sb.insert(i, r);
             }
         }
 
@@ -898,11 +1024,11 @@ public class JSVGCanvas extends JSVGComponent {
          * Checks if there is a peer element of a given type.  This returns the
          * first occurence of the given type or null if none is found.
          */
-        public Element getPeerWithTag(Element elt,
+        public Element getPeerWithTag(Element parent,
                                       String nameSpaceURI,
                                       String localName) {
 
-            Element p = (Element)elt.getParentNode();
+            Element p = parent;
             if (p == null) {
                 return null;
             }
@@ -936,22 +1062,39 @@ public class JSVGCanvas extends JSVGComponent {
          * Sets the tool tip on the input element.
          */
         public void setToolTip(Element elt, String toolTip){
-            EventTarget target = (EventTarget)elt;
-            elt.normalize();
+            if (toolTipMap == null) {
+                toolTipMap = new WeakHashMap();
+            }
+            if (toolTipDocs == null) {
+                toolTipDocs = new WeakHashMap();
+            }
+            SVGDocument doc = (SVGDocument)elt.getOwnerDocument();
+            if (toolTipDocs.put(doc, MAP_TOKEN) == null) {
+                NodeEventTarget root;
+                root = (NodeEventTarget)doc.getRootElement();
+                // On mouseover, it sets the tooltip to the given value
+                root.addEventListenerNS(XMLConstants.XML_EVENTS_NAMESPACE_URI,
+                                        SVGConstants.SVG_EVENT_MOUSEOVER,
+                                        toolTipListener,
+                                        false, null);
+                // On mouseout, it removes the tooltip
+                root.addEventListenerNS(XMLConstants.XML_EVENTS_NAMESPACE_URI,
+                                        SVGConstants.SVG_EVENT_MOUSEOUT,
+                                        toolTipListener,
+                                        false, null);
+            }
 
-            // On mouseover, set the tooltip to the title value
-            target.addEventListener(SVGConstants.SVG_EVENT_MOUSEOVER,
-                                    new ToolTipModifier(toolTip),
-                                    false);
+            toolTipMap.put(elt, toolTip);
 
-            // On mouseout, remove the tooltip
-            target.addEventListener(SVGConstants.SVG_EVENT_MOUSEOUT,
-                                    new ToolTipModifier(null),
-                                    false);
+            if (elt == lastTarget)
+                EventQueue.invokeLater(new ToolTipRunnable(toolTip));
+        }
 
-            if (locationListener == null) {
-                locationListener = new LocationListener();
-                addMouseMotionListener(locationListener);
+        public void removeToolTip(Element elt) {
+            if (toolTipMap != null)
+                toolTipMap.remove(elt);
+            if (lastTarget == elt) { // clear ToolTip.
+                EventQueue.invokeLater(new ToolTipRunnable(null));
             }
         }
 
@@ -967,7 +1110,7 @@ public class JSVGCanvas extends JSVGComponent {
                 JDialog dialog =
                     pane.createDialog(JSVGCanvas.this, "ERROR");
                 dialog.setModal(false);
-                dialog.show(); // Safe to be called from any thread
+                dialog.setVisible(true); // Safe to be called from any thread
             }
         }
 
@@ -982,7 +1125,7 @@ public class JSVGCanvas extends JSVGComponent {
                     new JErrorPane(ex, JOptionPane.ERROR_MESSAGE);
                 JDialog dialog = pane.createDialog(JSVGCanvas.this, "ERROR");
                 dialog.setModal(false);
-                dialog.show(); // Safe to be called from any thread
+                dialog.setVisible(true); // Safe to be called from any thread
             }
         }
     }
@@ -991,6 +1134,23 @@ public class JSVGCanvas extends JSVGComponent {
     // Tooltip
     // ----------------------------------------------------------------------
 
+    /**
+     * Sets the time and element of the last tool tip event handled.
+     */
+    public void setLastToolTipEvent(long t, EventTarget et) {
+        lastToolTipEventTimeStamp = t;
+        lastToolTipEventTarget = et;
+    }
+
+    /**
+     * Checks if the specified event time and element are the same
+     * as the last tool tip event.
+     */
+    public boolean matchLastToolTipEvent(long t, EventTarget et) {
+        return lastToolTipEventTimeStamp == t
+            && lastToolTipEventTarget == et;
+    }
+
     /**
      * Helper class. Simply keeps track of the last known mouse
      * position over the canvas.
@@ -999,6 +1159,10 @@ public class JSVGCanvas extends JSVGComponent {
 
         protected int lastX, lastY;
 
+        public LocationListener () {
+            lastX = 0; lastY = 0;
+        }
+
         public void mouseMoved(MouseEvent evt) {
             lastX = evt.getX();
             lastY = evt.getY();
@@ -1014,47 +1178,106 @@ public class JSVGCanvas extends JSVGComponent {
     }
 
     /**
-     * Sets a specific tooltip on the JSVGCanvas when a given event occurs. This
-     * listener is used in the handleElement method to set, remove or modify the
-     * JSVGCanvas tooltip on mouseover and on mouseout.<br/>
+     * Sets a specific tooltip on the JSVGCanvas when a given event occurs.
+     * This listener is used in the handleElement method to set, remove or
+     * modify the JSVGCanvas tooltip on mouseover and on mouseout.<br/>
      *
      * Because we are on a single <tt>JComponent</tt> we trigger an artificial
-     * <tt>MouseEvent</tt> when the toolTip is set to a non-null value, so as to
-     * make sure it will show after the <tt>ToolTipManager</tt>'s default delay.
+     * <tt>MouseEvent</tt> when the toolTip is set to a non-null value, so as
+     * to make sure it will show after the <tt>ToolTipManager</tt>'s default
+     * delay.
      */
     protected class ToolTipModifier implements EventListener {
         /**
-         * Value of the toolTip
+         * The CanvasUserAgent used to track the last tool tip event.
          */
-        protected String toolTip;
+        protected CanvasUserAgent canvasUserAgent;
 
         /**
-         * @param toolTip value to which the JSVGCanvas should be
-         *        set when the event occurs.
+         * Creates a new ToolTipModifier object.
          */
-        public ToolTipModifier(String toolTip){
-            this.toolTip = toolTip;
+        public ToolTipModifier() {
         }
 
         public void handleEvent(Event evt){
-            EventQueue.invokeLater(new Runnable() {
-                    public void run() {
-                        setToolTipText(toolTip);
+            // Don't set the tool tip if another ToolTipModifier
+            // has already handled this event (as it will have been
+            // a higher priority tool tip).
+            if (matchLastToolTipEvent(evt.getTimeStamp(), evt.getTarget())) {
+                return;
+            }
+            setLastToolTipEvent(evt.getTimeStamp(), evt.getTarget());
+            EventTarget prevLastTarget = lastTarget;
+            if (SVGConstants.SVG_EVENT_MOUSEOVER.equals(evt.getType())) {
+                lastTarget = evt.getTarget();
+            } else if (SVGConstants.SVG_EVENT_MOUSEOUT.equals(evt.getType())) {
+                // related target is one it is entering or null.
+                org.w3c.dom.events.MouseEvent mouseEvt;
+                mouseEvt = ((org.w3c.dom.events.MouseEvent)evt);
+                lastTarget = mouseEvt.getRelatedTarget();
+            }
 
-                        if (toolTip != null) {
-                            MouseEvent e = new MouseEvent
-                                (JSVGCanvas.this,
-                                 MouseEvent.MOUSE_ENTERED,
-                                 System.currentTimeMillis(),
-                                 0,
-                                 locationListener.getLastX(),
-                                 locationListener.getLastY(),
-                                 0,
-                                 false);
-                            ToolTipManager.sharedInstance().mouseEntered(e);
-                        }
+            if (toolTipMap != null) {
+                Element e = (Element)lastTarget;
+                Object o = null;
+                while (e != null) {
+                    // Search the parents of the current node for ToolTips.
+                    o = toolTipMap.get(e);
+                    if (o != null) {
+                        break;
                     }
-                });
+                    e = CSSEngine.getParentCSSStylableElement(e);
+                }
+                final String theToolTip = (String)o;
+                if (prevLastTarget != lastTarget)
+                    EventQueue.invokeLater(new ToolTipRunnable(theToolTip));
+            }
+        }
+    }
+
+    protected class ToolTipRunnable implements Runnable {
+        String theToolTip;
+        public ToolTipRunnable(String toolTip) {
+            this.theToolTip = toolTip;
+        }
+
+        public void run() {
+            setToolTipText(theToolTip);
+
+            MouseEvent e;
+            if (theToolTip != null) {
+                e = new MouseEvent
+                    (JSVGCanvas.this,
+                     MouseEvent.MOUSE_ENTERED,
+                     System.currentTimeMillis(),
+                     0,
+                     locationListener.getLastX(),
+                     locationListener.getLastY(),
+                     0,
+                     false);
+                ToolTipManager.sharedInstance().mouseEntered(e);
+                e = new MouseEvent
+                    (JSVGCanvas.this,
+                     MouseEvent.MOUSE_MOVED,
+                     System.currentTimeMillis(),
+                     0,
+                     locationListener.getLastX(),
+                     locationListener.getLastY(),
+                     0,
+                     false);
+                ToolTipManager.sharedInstance().mouseMoved(e);
+            } else {
+                e = new MouseEvent
+                    (JSVGCanvas.this,
+                     MouseEvent.MOUSE_MOVED,
+                     System.currentTimeMillis(),
+                     0,
+                     locationListener.getLastX(),
+                     locationListener.getLastY(),
+                     0,
+                     false);
+                ToolTipManager.sharedInstance().mouseMoved(e);
+            }
         }
     }
 }
