@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.test.svg;
 
 import java.io.BufferedInputStream;
@@ -63,6 +31,9 @@ import java.io.StringWriter;
 import java.net.URL;
 import java.net.MalformedURLException;
 
+import java.util.Iterator;
+import java.util.LinkedList;
+import java.util.List;
 import java.util.Locale;
 import java.util.ResourceBundle;
 
@@ -76,11 +47,10 @@ import java.awt.image.ColorModel;
 import org.apache.batik.ext.awt.image.GraphicsUtil;
 
 import org.apache.batik.ext.awt.image.spi.ImageTagRegistry;
+import org.apache.batik.ext.awt.image.spi.ImageWriter;
+import org.apache.batik.ext.awt.image.spi.ImageWriterRegistry;
 import org.apache.batik.ext.awt.image.renderable.Filter;
 
-import org.apache.batik.ext.awt.image.codec.PNGImageEncoder;
-import org.apache.batik.ext.awt.image.codec.PNGEncodeParam;
-
 import org.apache.batik.util.ParsedURL;
 
 import org.apache.batik.test.AbstractTest;
@@ -89,20 +59,20 @@ import org.apache.batik.test.TestReport;
 
 /**
  * Checks for regressions in rendering a specific SVG document.
- * The <tt>Test</tt> will rasterize and SVG document and 
- * compare it to a reference image. The test passes if the 
+ * The <tt>Test</tt> will rasterize and SVG document and
+ * compare it to a reference image. The test passes if the
  * rasterized SVG and the reference image match exactly (i.e.,
  * all pixel values are the same).
  *
  * @author <a href="mailto:vhardy@apache.lorg">Vincent Hardy</a>
- * @version $Id: AbstractRenderingAccuracyTest.java,v 1.4 2003/08/09 16:58:43 deweese Exp $
+ * @version $Id: AbstractRenderingAccuracyTest.java 581141 2007-10-02 05:36:50Z cam $
  */
 public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
     /**
      * Error when temp file cannot be created
      * {0} = IOException message
      */
-    public static final String ERROR_CANNOT_CREATE_TEMP_FILE        
+    public static final String ERROR_CANNOT_CREATE_TEMP_FILE
         = "SVGRenderingAccuracyTest.error.cannot.create.temp.file";
 
     /**
@@ -110,7 +80,7 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
      * {0} = temp file's cannonical path
      * {1} = IOException message
      */
-    public static final String ERROR_CANNOT_CREATE_TEMP_FILE_STREAM 
+    public static final String ERROR_CANNOT_CREATE_TEMP_FILE_STREAM
         = "SVGRenderingAccuracyTest.error.cannot.create.temp.file.stream";
 
     /**
@@ -118,7 +88,7 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
      * {0} = URI of the reference image
      * {1} = IOException message
      */
-    public static final String ERROR_CANNOT_OPEN_REFERENCE_IMAGE    
+    public static final String ERROR_CANNOT_OPEN_REFERENCE_IMAGE
         = "SVGRenderingAccuracyTest.error.cannot.open.reference.image";
 
     /**
@@ -126,31 +96,31 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
      * {0} = Cannonical path of the temp generated image
      * {1} = IOException message
      */
-    public static final String ERROR_CANNOT_OPEN_GENERATED_IMAGE    
+    public static final String ERROR_CANNOT_OPEN_GENERATED_IMAGE
         = "SVGRenderingAccuracyTest.error.cannot.open.genereted.image";
 
     /**
-     * Error when there is an IOException while comparing the 
+     * Error when there is an IOException while comparing the
      * two reference raster image with the new raster image built
      * from the SVG.
      * {0} = URI of the reference image
      * {1} = Connical path for the temp generated image
      * {2} = IOException message.
      */
-    public static final String ERROR_ERROR_WHILE_COMPARING_FILES    
+    public static final String ERROR_ERROR_WHILE_COMPARING_FILES
         = "SVGRenderingAccuracyTest.error.while.comparing.files";
 
     /**
      * Error when the generated image from the SVG file differs from
      * the reference image.
      */
-    public static final String ERROR_SVG_RENDERING_NOT_ACCURATE     
+    public static final String ERROR_SVG_RENDERING_NOT_ACCURATE
         = "SVGRenderingAccuracyTest.error.svg.rendering.not.accurate";
 
     /**
      * Entry describing the error
      */
-    public static final String ENTRY_KEY_ERROR_DESCRIPTION 
+    public static final String ENTRY_KEY_ERROR_DESCRIPTION
         = "SVGRenderingAccuracyTest.entry.key.error.description";
 
     /**
@@ -191,38 +161,38 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
 
     /**
      * Message expressing that the variation URL could not be open
-     * {0} : URL 
+     * {0} : URL
      */
-    public static final String COULD_NOT_OPEN_VARIATION_URL 
+    public static final String COULD_NOT_OPEN_VARIATION_URL
         = "SVGRenderingAccuracyTest.message.warning.could.not.open.variation.url";
 
     /**
      * The gui resources file name
      */
-    public final static String CONFIGURATION_RESOURCES =
+    public static final String CONFIGURATION_RESOURCES =
         "org.apache.batik.test.svg.resources.Configuration";
 
     /**
      * Suffix used for comparison images
      */
-    public final static String IMAGE_TYPE_COMPARISON = "_cmp";
+    public static final String IMAGE_TYPE_COMPARISON = "_cmp";
 
     /**
      * Suffix used for diff images
      */
-    public final static String IMAGE_TYPE_DIFF = "_diff";
+    public static final String IMAGE_TYPE_DIFF = "_diff";
 
     /**
      * Suffix used for saved images (e.g., comparison and diff images)
      */
-    public final static String IMAGE_FILE_EXTENSION = ".png";
+    public static final String IMAGE_FILE_EXTENSION = ".png";
 
     /**
      * The configuration resource bundle
      */
     protected static ResourceBundle configuration;
     static {
-        configuration = ResourceBundle.getBundle(CONFIGURATION_RESOURCES, 
+        configuration = ResourceBundle.getBundle(CONFIGURATION_RESOURCES,
                                                  Locale.getDefault());
     }
 
@@ -230,11 +200,11 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
      * Prefix for the temporary files created by Tests
      * of this class
      */
-    public static final String TEMP_FILE_PREFIX 
+    public static final String TEMP_FILE_PREFIX
         = configuration.getString("temp.file.prefix");
 
     /**
-     * Suffix for the temporary files created by 
+     * Suffix for the temporary files created by
      * Tests of this class
      */
     public static final String TEMP_FILE_SUFFIX
@@ -251,20 +221,20 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
     protected URL refImgURL;
 
     /**
-     * The URL of a file containing an 'accepted' 
+     * A list of {@link URL}s of files containing an 'accepted'
      * variation from the reference image.
      */
-    protected URL variationURL;
+    protected List variationURLs;
 
     /**
-     * The File where the newly computed variation 
-     * should be saved if different from the 
+     * The File where the newly computed variation
+     * should be saved if different from the
      * variationURL
      */
     protected File saveVariation;
 
     /**
-     * The File where the candidate reference 
+     * The File where the candidate reference
      * should be saved if there is not candidate reference
      * or if it cannot be opened.
      */
@@ -325,7 +295,7 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
         this.svgURL = resolveURL(svgURL);
         this.refImgURL = resolveURL(refImgURL);
     }
-    
+
     /**
      * Resolves the input string as follows.
      * + First, the string is interpreted as a file description.
@@ -344,7 +314,7 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
                 throw new IllegalArgumentException();
             }
         }
-        
+
         // url is not a file. It must be a regular URL...
         try{
             return new URL(url);
@@ -365,16 +335,21 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
         return saveVariation;
     }
 
-    public String getVariationURL(){
-        return variationURL.toString();
+    public String[] getVariationURLs() {
+        if (variationURLs != null) {
+            return (String[]) variationURLs.toArray(new String[0]);
+        }
+        return null;
     }
 
     /**
-     * Sets the URL where an acceptable variation fron the reference 
-     * image can be found.
+     * Adds a URL for an acceptable variation from the reference image.
      */
-    public void setVariationURL(String variationURL){
-        this.variationURL = resolveURL(variationURL);
+    public void addVariationURL(String variationURL) {
+        if (this.variationURLs == null) {
+            this.variationURLs = new LinkedList();
+        }
+        this.variationURLs.add(resolveURL(variationURL));
     }
 
     /**
@@ -389,7 +364,7 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
     }
 
     /**
-     * Returns this <tt>Test</tt>'s name. The name is the 
+     * Returns this <tt>Test</tt>'s name. The name is the
      * URL of the SVG being rendered.
      */
     public String getName(){
@@ -400,7 +375,7 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
     }
 
     /**
-     * Requests this <tt>Test</tt> to run and produce a 
+     * Requests this <tt>Test</tt> to run and produce a
      * report.
      *
      */
@@ -415,7 +390,7 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
                 candidateReference.delete();
             }
         }
-                    
+
 
         //
         // Render the SVG image into a raster. We call an
@@ -432,10 +407,10 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
                                               null);
         }catch(IOException e){
             report.setErrorCode(ERROR_CANNOT_CREATE_TEMP_FILE);
-            report.setDescription(new TestReport.Entry[] { 
+            report.setDescription(new TestReport.Entry[] {
                 new TestReport.Entry
                 (Messages.formatMessage(ENTRY_KEY_ERROR_DESCRIPTION, null),
-                 Messages.formatMessage(ERROR_CANNOT_CREATE_TEMP_FILE, 
+                 Messages.formatMessage(ERROR_CANNOT_CREATE_TEMP_FILE,
                                         new Object[]{e.getMessage()}))
             });
             report.setPassed(false);
@@ -465,8 +440,7 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
         // encoding failed and we should return that report.
         {
             TestReport encodeTR = encode(svgURL, tmpFileOS);
-            if ((encodeTR != null) && 
-                (encodeTR.hasPassed() == false)) {
+            if ((encodeTR != null) && ! encodeTR.hasPassed() ) {
                 tmpFile.deleteOnExit();
                 return encodeTR;
             }
@@ -485,14 +459,14 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
                 new TestReport.Entry
                 (Messages.formatMessage(ENTRY_KEY_ERROR_DESCRIPTION, null),
                  Messages.formatMessage(ERROR_CANNOT_OPEN_REFERENCE_IMAGE,
-                                        new Object[]{refImgURL.toString(), 
-                                                     e.getMessage()})) 
+                                        new Object[]{refImgURL.toString(),
+                                                     e.getMessage()}))
                 });
             report.setPassed(false);
             // Try and save tmp file as a candidate variation
             if (candidateReference == null){
                 tmpFile.delete();
-            } 
+            }
             return report;
         }
 
@@ -504,7 +478,7 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
                 new TestReport.Entry
                 (Messages.formatMessage(ENTRY_KEY_ERROR_DESCRIPTION, null),
                  Messages.formatMessage(ERROR_CANNOT_OPEN_GENERATED_IMAGE,
-                                        new Object[]{tmpFile.getAbsolutePath(), 
+                                        new Object[]{tmpFile.getAbsolutePath(),
                                                      e.getMessage()}))});
             report.setPassed(false);
             tmpFile.delete();
@@ -520,7 +494,7 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
                 new TestReport.Entry
                 (Messages.formatMessage(ENTRY_KEY_ERROR_DESCRIPTION, null),
                  Messages.formatMessage(ERROR_ERROR_WHILE_COMPARING_FILES,
-                                        new Object[]{refImgURL.toString(), 
+                                        new Object[]{refImgURL.toString(),
                                                      tmpFile.getAbsolutePath(),
                                                      e.getMessage()}))});
             report.setPassed(false);
@@ -554,32 +528,39 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
             // If there is an accepted variation, check if it equals the
             // computed difference.
             //
-            if(variationURL != null) {
-                File tmpDiff = imageToFile(diff, IMAGE_TYPE_DIFF);
-                
-                InputStream variationURLStream = null;
-                try{
-                    variationURLStream = variationURL.openStream();
-                }catch(IOException e){
-                    // Could not open variationURL stream. Just trace that
-                    System.err.println(Messages.formatMessage(COULD_NOT_OPEN_VARIATION_URL,
-                                                              new Object[]{variationURL.toString()}));
-                }
-                
-                if(variationURLStream != null){
-                    InputStream refDiffStream =
-                        new BufferedInputStream(variationURLStream);
-                    
-                    InputStream tmpDiffStream =
-                        new BufferedInputStream(new FileInputStream(tmpDiff));
-                    
-                    if(compare(refDiffStream, tmpDiffStream)){
-                        // We accept the generated result.
-                        accurate = true;
+            if (variationURLs != null) {
+                Iterator it = variationURLs.iterator();
+                while (it.hasNext()) {
+                    URL variationURL = (URL) it.next();
+                    File tmpDiff = imageToFile(diff, IMAGE_TYPE_DIFF);
+
+                    InputStream variationURLStream = null;
+                    try {
+                        variationURLStream = variationURL.openStream();
+                    } catch (IOException e) {
+                        // Could not open variationURL stream. Just trace that
+                        System.err.println
+                            (Messages.formatMessage
+                                (COULD_NOT_OPEN_VARIATION_URL,
+                                 new Object[] { variationURL.toString() }));
+                    }
+
+                    if (variationURLStream != null) {
+                        InputStream refDiffStream =
+                            new BufferedInputStream(variationURLStream);
+
+                        InputStream tmpDiffStream =
+                            new BufferedInputStream
+                                (new FileInputStream(tmpDiff));
+
+                        if (compare(refDiffStream, tmpDiffStream)) {
+                            // We accept the generated result.
+                            accurate = true;
+                        }
                     }
                 }
             }
-            
+
             if (accurate) {
                 //
                 // Yahooooooo! everything worked out well, at least
@@ -592,20 +573,20 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
             System.err.println(">>>>>>>>>>>>>>>>>>>>>> "+
                                "Rendering is not accurate");
             if(saveVariation != null){
-                // There is a computed variation different from the 
-                // referenced variation and there is a place where the new 
+                // There is a computed variation different from the
+                // referenced variation and there is a place where the new
                 // variation should be saved.
                 saveImage(diff, saveVariation);
             }
-                
+
             // Build two images:
             // a. One with the reference image and the newly generated image
-            // b. One with the difference between the two images and the set 
+            // b. One with the difference between the two images and the set
             //    of different pixels.
             BufferedImage cmp = makeCompareImage(ref, gen);
             File cmpFile = imageToFile(cmp, IMAGE_TYPE_COMPARISON);
             File diffFile = imageToFile(diff, IMAGE_TYPE_DIFF);
-            
+
             report.setErrorCode(ERROR_SVG_RENDERING_NOT_ACCURATE);
             report.setDescription(new TestReport.Entry[]{
                 new TestReport.Entry
@@ -621,23 +602,23 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
             report.setErrorCode(ERROR_SVG_RENDERING_NOT_ACCURATE);
             StringWriter trace = new StringWriter();
             e.printStackTrace(new PrintWriter(trace));
-            
+
             report.setDescription(new TestReport.Entry[]{
                 new TestReport.Entry
                 (Messages.formatMessage(ENTRY_KEY_ERROR_DESCRIPTION, null),
                  Messages.formatMessage(ERROR_SVG_RENDERING_NOT_ACCURATE, null)),
                 new TestReport.Entry
                 (Messages.formatMessage(ENTRY_KEY_INTERNAL_ERROR, null),
-                 Messages.formatMessage(COULD_NOT_GENERATE_COMPARISON_IMAGES, 
+                 Messages.formatMessage(COULD_NOT_GENERATE_COMPARISON_IMAGES,
                                         new Object[]{e.getClass().getName(),
                                                      e.getMessage(),
                                                      trace.toString()})) });
         }
-        
+
         if (candidateReference == null){
             tmpFile.delete();
         }
-            
+
         report.setPassed(false);
         return report;
     }
@@ -668,18 +649,22 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
         if(!imgFile.exists()){
             imgFile.createNewFile();
         }
-        saveImage(img, new FileOutputStream(imgFile));
+        OutputStream out = new FileOutputStream(imgFile);
+        try {
+            saveImage(img, out);
+        } finally {
+            out.close();
+        }
     }
 
     /**
      * Saves an image in a given File
      */
     protected void saveImage(BufferedImage img, OutputStream os)
-        throws IOException {
-        PNGImageEncoder encoder = new PNGImageEncoder
-            (os, PNGEncodeParam.getDefaultEncodeParam(img));
-        
-        encoder.encode(img);
+            throws IOException {
+        ImageWriter writer = ImageWriterRegistry.getInstance()
+            .getWriterFor("image/png");
+        writer.writeImage(img, os);
     }
 
     /**
@@ -731,7 +716,7 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
             ColorModel cm = gen.getColorModel();
             cm = GraphicsUtil.coerceData(genWR, cm, false);
         }
-        
+
         if (!refPre) {
             ColorModel cm = ref.getColorModel();
             cm = GraphicsUtil.coerceData(refWR, cm, false);
@@ -743,7 +728,7 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
     /**
      * Loads an image from a File
      */
-    protected BufferedImage getImage(File file) 
+    protected BufferedImage getImage(File file)
         throws Exception {
         return getImage(file.toURL());
     }
@@ -751,7 +736,7 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
     /**
      * Loads an image from a URL
      */
-    protected BufferedImage getImage(URL url) 
+    protected BufferedImage getImage(URL url)
         throws IOException {
         ImageTagRegistry reg = ImageTagRegistry.getRegistry();
         Filter filt = reg.readURL(new ParsedURL(url));
@@ -765,7 +750,7 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
             throw new IOException(Messages.formatMessage
                                   (COULD_NOT_LOAD_IMAGE,
                                    new Object[]{url.toString()}));
-        
+
         BufferedImage img = new BufferedImage(red.getWidth(),
                                               red.getHeight(),
                                               BufferedImage.TYPE_INT_ARGB);
@@ -798,9 +783,9 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
      * Creates a File into which the input image is
      * saved.
      * If there is a "file" component in the SVG url,
-     * then a temporary file is created with that 
-     * name and the imageType suffix in the temp 
-     * directory of the test-reports directory. 
+     * then a temporary file is created with that
+     * name and the imageType suffix in the temp
+     * directory of the test-reports directory.
      */
     protected File imageToFile(BufferedImage img,
                                String imageType)
@@ -814,15 +799,11 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
         else{
             imageFile = makeRandomFileName(imageType);
         }
-        
+
         imageFile.deleteOnExit();
 
-        PNGImageEncoder encoder = new PNGImageEncoder
-            (new FileOutputStream(imageFile),
-             PNGEncodeParam.getDefaultEncodeParam(img));
-        
-        encoder.encode(img);
-        
+        saveImage(img, imageFile);
+
         return imageFile;
     }
 
@@ -857,7 +838,7 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
                  imageType + IMAGE_FILE_EXTENSION);
         }
     }
-    
+
     protected File getNextTempFileName(String fileName){
         File f = new File(getTempDirectory(), fileName);
         if(!f.exists()){
@@ -866,9 +847,9 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
         else{
             return getNextTempFileName(fileName,
                                        1);
-        } 
+        }
     }
-    
+
     protected File getNextTempFileName(String fileName,
                                        int instance){
         // First, create a 'versioned' file name
@@ -878,7 +859,7 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
             iFileName = fileName.substring(0, n) + instance
                 + fileName.substring(n, fileName.length());
         }
-        
+
         File r = new File(getTempDirectory(), iFileName);
         if(!r.exists()){
             return r;
@@ -888,14 +869,14 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
                                        instance + 1);
         }
     }
-    
+
     /**
      * Creates a temporary File into which the input image is
      * saved.
      */
     protected File makeRandomFileName(String imageType)
         throws IOException {
-        
+
         return File.createTempFile(TEMP_FILE_PREFIX,
                                    TEMP_FILE_SUFFIX + imageType,
                                    null);
